from collections.abc import Iterable, Iterator, Mapping, MutableMapping
from typing import Any

from . import Command, SetuptoolsDeprecationWarning
from ._distutils.dist import Distribution as _Distribution

__all__ = ["Distribution"]

class Distribution(_Distribution):
    def patch_missing_pkg_info(self, attrs: Mapping[str, Any]) -> None: ...
    src_root: str | None
    dependency_links: list[str]
    setup_requires: list[str]
    def __init__(self, attrs: MutableMapping[str, Any] | None = None) -> None: ...
    def warn_dash_deprecation(self, opt: str, section: str) -> str: ...
    def make_option_lowercase(self, opt: str, section: str) -> str: ...
    def parse_config_files(self, filenames: Iterable[str] | None = None, ignore_option_errors: bool = False) -> None: ...
    def fetch_build_eggs(self, requires: str | Iterable[str]): ...
    def get_egg_cache_dir(self) -> str: ...
    def fetch_build_egg(self, req): ...
    def get_command_class(self, command: str) -> type[Command]: ...
    def include(self, **attrs) -> None: ...
    def exclude_package(self, package: str) -> None: ...
    def has_contents_for(self, package: str) -> bool | None: ...
    def exclude(self, **attrs) -> None: ...
    def get_cmdline_options(self) -> dict[str, dict[str, str | None]]: ...
    def iter_distribution_names(self) -> Iterator[str]: ...
    def handle_display_options(self, option_order): ...

class DistDeprecationWarning(SetuptoolsDeprecationWarning): ...



from collections.abc import Callable, Iterable, Iterator, Sequence
from typing import Any
from typing_extensions import Self, TypeAlias

from wtforms.fields.core import Field, _Filter, _FormT, _Validator, _Widget
from wtforms.form import BaseForm
from wtforms.meta import DefaultMeta, _SupportsGettextAndNgettext

# technically this allows a list, but we're more strict for type safety
_Choice: TypeAlias = tuple[Any, str] | tuple[Any, str, dict[str, Any]]
# it's too difficult to get type safety here due to to nested partially invariant collections
_GroupedChoices: TypeAlias = dict[str, Any]  # Any should be Collection[_Choice]
_FullChoice: TypeAlias = tuple[Any, str, bool, dict[str, Any]]  # value, label, selected, render_kw
_FullGroupedChoices: TypeAlias = tuple[str, Iterable[_FullChoice]]
_Option: TypeAlias = SelectFieldBase._Option

class SelectFieldBase(Field):
    option_widget: _Widget[_Option]
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        option_widget: _Widget[_Option] | None = None,
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: object | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...
    def iter_choices(self) -> Iterator[_FullChoice]: ...
    def has_groups(self) -> bool: ...
    def iter_groups(self) -> Iterator[_FullGroupedChoices]: ...
    def __iter__(self) -> Iterator[_Option]: ...

    class _Option(Field):
        checked: bool

class SelectField(SelectFieldBase):
    coerce: Callable[[Any], Any]
    choices: list[_Choice] | _GroupedChoices
    validate_choice: bool
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        coerce: Callable[[Any], Any] = ...,
        choices: Iterable[_Choice] | _GroupedChoices | Callable[[], Iterable[_Choice] | _GroupedChoices] | None = None,
        validate_choice: bool = True,
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: object | None = None,
        widget: _Widget[Self] | None = None,
        option_widget: _Widget[_Option] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...
    def iter_choices(self) -> Iterator[_FullChoice]: ...
    def has_groups(self) -> bool: ...
    def iter_groups(self) -> Iterator[_FullGroupedChoices]: ...

class SelectMultipleField(SelectField):
    data: list[Any] | None

class RadioField(SelectField): ...



from typing import ClassVar

from .core import UnitedStates

class Pennsylvania(UnitedStates):
    include_good_friday: ClassVar[bool]
    include_thanksgiving_friday: ClassVar[bool]
    include_election_day_every_year: ClassVar[bool]



from _typeshed import Incomplete

class Links:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self, next: Incomplete | None = None, _self: Incomplete | None = None, prev: Incomplete | None = None
    ) -> None: ...
    @property
    def next(self): ...
    @next.setter
    def next(self, next) -> None: ...
    @property
    def prev(self): ...
    @prev.setter
    def prev(self, prev) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from typing import Any, ClassVar, Literal

FIELDNAME: Any

class Limit:
    offset: Any
    count: Any
    def __init__(self, offset: int = 0, count: int = 0) -> None: ...
    def build_args(self): ...

class Reducer:
    NAME: ClassVar[None]
    def __init__(self, *args) -> None: ...
    def alias(self, alias): ...
    @property
    def args(self): ...

class SortDirection:
    DIRSTRING: ClassVar[str | None]
    field: Any
    def __init__(self, field) -> None: ...

class Asc(SortDirection):
    DIRSTRING: ClassVar[Literal["ASC"]]

class Desc(SortDirection):
    DIRSTRING: ClassVar[Literal["DESC"]]

class AggregateRequest:
    def __init__(self, query: str = "*") -> None: ...
    def load(self, *fields): ...
    def group_by(self, fields, *reducers): ...
    def apply(self, **kwexpr): ...
    def limit(self, offset, num): ...
    def sort_by(self, *fields, **kwargs): ...
    def filter(self, expressions): ...
    def with_schema(self): ...
    def verbatim(self): ...
    def cursor(self, count: int = 0, max_idle: float = 0.0): ...
    def build_args(self): ...

class Cursor:
    cid: Any
    max_idle: int
    count: int
    def __init__(self, cid) -> None: ...
    def build_args(self): ...

class AggregateResult:
    rows: Any
    cursor: Any
    schema: Any
    def __init__(self, rows, cursor, schema) -> None: ...



from collections.abc import Iterable

DEFAULT_MOUNT_POINT: str
ALLOWED_CREDS_ENDPOINTS: Iterable[str]
ALLOWED_CREDS_TYPES: Iterable[str]
ALLOWED_IAM_ALIAS_TYPES: Iterable[str]
ALLOWED_EC2_ALIAS_TYPES: Iterable[str]



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def betweenness_centrality(
    G,
    k: Incomplete | None = None,
    normalized: bool = True,
    weight: Incomplete | None = None,
    endpoints: bool = False,
    seed: Incomplete | None = None,
): ...
@_dispatch
def edge_betweenness_centrality(
    G, k: Incomplete | None = None, normalized: bool = True, weight: Incomplete | None = None, seed: Incomplete | None = None
): ...



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class BackupService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def get_backup_kv(self, **kwargs): ...
    def get_backup_kv_with_http_info(self, **kwargs): ...
    async def get_backup_kv_async(self, **kwargs): ...
    def get_backup_metadata(self, **kwargs): ...
    def get_backup_metadata_with_http_info(self, **kwargs): ...
    async def get_backup_metadata_async(self, **kwargs): ...
    def get_backup_shard_id(self, shard_id, **kwargs): ...
    def get_backup_shard_id_with_http_info(self, shard_id, **kwargs): ...
    async def get_backup_shard_id_async(self, shard_id, **kwargs): ...



import enum
from _typeshed import Incomplete, StrOrBytesPath, SupportsWrite
from collections.abc import Callable
from socket import AF_INET6 as AF_INET6, AddressFamily, SocketKind
from typing import Any, Literal, NamedTuple, TypeVar, overload

POSIX: bool
WINDOWS: bool
LINUX: bool
MACOS: bool
OSX: bool
FREEBSD: bool
OPENBSD: bool
NETBSD: bool
BSD: bool
SUNOS: bool
AIX: bool

STATUS_RUNNING: Literal["running"]
STATUS_SLEEPING: Literal["sleeping"]
STATUS_DISK_SLEEP: Literal["disk-sleep"]
STATUS_STOPPED: Literal["stopped"]
STATUS_TRACING_STOP: Literal["tracing-stop"]
STATUS_ZOMBIE: Literal["zombie"]
STATUS_DEAD: Literal["dead"]
STATUS_WAKE_KILL: Literal["wake-kill"]
STATUS_WAKING: Literal["waking"]
STATUS_IDLE: Literal["idle"]
STATUS_LOCKED: Literal["locked"]
STATUS_WAITING: Literal["waiting"]
STATUS_SUSPENDED: Literal["suspended"]
STATUS_PARKED: Literal["parked"]

CONN_ESTABLISHED: str
CONN_SYN_SENT: str
CONN_SYN_RECV: str
CONN_FIN_WAIT1: str
CONN_FIN_WAIT2: str
CONN_TIME_WAIT: str
CONN_CLOSE: str
CONN_CLOSE_WAIT: str
CONN_LAST_ACK: str
CONN_LISTEN: str
CONN_CLOSING: str
CONN_NONE: str
NIC_DUPLEX_FULL: int
NIC_DUPLEX_HALF: int
NIC_DUPLEX_UNKNOWN: int

class NicDuplex(enum.IntEnum):
    NIC_DUPLEX_FULL: int
    NIC_DUPLEX_HALF: int
    NIC_DUPLEX_UNKNOWN: int

POWER_TIME_UNKNOWN: int
POWER_TIME_UNLIMITED: int

class BatteryTime(enum.IntEnum):
    POWER_TIME_UNKNOWN: int
    POWER_TIME_UNLIMITED: int

ENCODING: str
ENCODING_ERRS: str

class sswap(NamedTuple):
    total: int
    used: int
    free: int
    percent: float
    sin: int
    sout: int

class sdiskusage(NamedTuple):
    total: int
    used: int
    free: int
    percent: float

class sdiskio(NamedTuple):
    read_count: int
    write_count: int
    read_bytes: int
    write_bytes: int
    read_time: int
    write_time: int

class sdiskpart(NamedTuple):
    device: str
    mountpoint: str
    fstype: str
    opts: str
    maxfile: int
    maxpath: int

class snetio(NamedTuple):
    bytes_sent: int
    bytes_recv: int
    packets_sent: int
    packets_recv: int
    errin: int
    errout: int
    dropin: int
    dropout: int

class suser(NamedTuple):
    name: str
    terminal: str | None
    host: str | None
    started: float
    pid: str

class sconn(NamedTuple):
    fd: int
    family: AddressFamily
    type: SocketKind
    laddr: addr | tuple[()]
    raddr: addr | tuple[()]
    status: str
    pid: int | None

class snicaddr(NamedTuple):
    family: AddressFamily
    address: str
    netmask: str | None
    broadcast: str | None
    ptp: str | None

class snicstats(NamedTuple):
    isup: bool
    duplex: int
    speed: int
    mtu: int
    flags: str

class scpustats(NamedTuple):
    ctx_switches: int
    interrupts: int
    soft_interrupts: int
    syscalls: int

class scpufreq(NamedTuple):
    current: float
    min: float
    max: float

class shwtemp(NamedTuple):
    label: str
    current: float
    high: float | None
    critical: float | None

class sbattery(NamedTuple):
    percent: int
    secsleft: int
    power_plugged: bool

class sfan(NamedTuple):
    label: str
    current: int

class pcputimes(NamedTuple):
    user: float
    system: float
    children_user: float
    children_system: float

class popenfile(NamedTuple):
    path: str
    fd: int

class pthread(NamedTuple):
    id: int
    user_time: float
    system_time: float

class puids(NamedTuple):
    real: int
    effective: int
    saved: int

class pgids(NamedTuple):
    real: int
    effective: int
    saved: int

class pio(NamedTuple):
    read_count: int
    write_count: int
    read_bytes: int
    write_bytes: int

class pionice(NamedTuple):
    ioclass: int
    value: int

class pctxsw(NamedTuple):
    voluntary: int
    involuntary: int

class pconn(NamedTuple):
    fd: int
    family: AddressFamily
    type: SocketKind
    laddr: addr
    raddr: addr
    status: str

class addr(NamedTuple):
    ip: str
    port: int

conn_tmap: dict[str, tuple[list[AddressFamily], list[SocketKind]]]

class Error(Exception):
    __module__: str
    msg: Any
    def __init__(self, msg: str = ...) -> None: ...

class NoSuchProcess(Error):
    __module__: str
    pid: Any
    name: Any
    msg: Any
    def __init__(self, pid, name: Incomplete | None = None, msg: Incomplete | None = None) -> None: ...

class ZombieProcess(NoSuchProcess):
    __module__: str
    pid: Any
    ppid: Any
    name: Any
    msg: Any
    def __init__(
        self, pid, name: Incomplete | None = None, ppid: Incomplete | None = None, msg: Incomplete | None = None
    ) -> None: ...

class AccessDenied(Error):
    __module__: str
    pid: Any
    name: Any
    msg: Any
    def __init__(self, pid: Incomplete | None = None, name: Incomplete | None = None, msg: Incomplete | None = None) -> None: ...

class TimeoutExpired(Error):
    __module__: str
    seconds: Any
    pid: Any
    name: Any
    def __init__(self, seconds, pid: Incomplete | None = None, name: Incomplete | None = None) -> None: ...

_Func = TypeVar("_Func", bound=Callable[..., Any])

def usage_percent(used, total, round_: int | None = None) -> float: ...
def memoize(fun: _Func) -> _Func: ...
def memoize_when_activated(fun: _Func) -> _Func: ...
def isfile_strict(path: StrOrBytesPath) -> bool: ...
def path_exists_strict(path: StrOrBytesPath) -> bool: ...
def supports_ipv6() -> bool: ...
def parse_environ_block(data): ...
def sockfam_to_enum(num: int) -> AddressFamily: ...
def socktype_to_enum(num: int) -> SocketKind: ...
@overload
def conn_to_ntuple(fd: int, fam: int, type_: int, laddr, raddr, status: str, status_map, pid: int) -> sconn: ...
@overload
def conn_to_ntuple(fd: int, fam: int, type_: int, laddr, raddr, status: str, status_map, pid: None = None) -> pconn: ...
def deprecated_method(replacement: str) -> Callable[[_Func], _Func]: ...

class _WrapNumbers:
    lock: Any
    cache: Any
    reminders: Any
    reminder_keys: Any
    def __init__(self) -> None: ...
    def run(self, input_dict, name): ...
    def cache_clear(self, name: Incomplete | None = None) -> None: ...
    def cache_info(self): ...

def wrap_numbers(input_dict, name: str): ...
def open_binary(fname): ...
def open_text(fname): ...
def cat(fname, fallback=..., _open=...): ...
def bcat(fname, fallback=...): ...
def bytes2human(n: int, format: str = "%(value).1f%(symbol)s") -> str: ...
def get_procfs_path() -> str: ...
def term_supports_colors(file: SupportsWrite[str] = ...) -> bool: ...
def hilite(s: str, color: str | None = None, bold: bool = False) -> str: ...
def print_color(s: str, color: str | None = None, bold: bool = False, file: SupportsWrite[str] = ...) -> None: ...
def debug(msg) -> None: ...



import sys
from _typeshed import Incomplete
from collections.abc import Sequence
from socket import socket
from typing import Any

from waitress import wasyncore
from waitress.adjustments import Adjustments
from waitress.channel import HTTPChannel
from waitress.task import Task, ThreadedTaskDispatcher

def create_server(
    application: Any,
    map: Incomplete | None = None,
    _start: bool = True,
    _sock: socket | None = None,
    _dispatcher: ThreadedTaskDispatcher | None = None,
    **kw: Any,
) -> MultiSocketServer | BaseWSGIServer: ...

class MultiSocketServer:
    asyncore: Any
    adj: Adjustments
    map: Any
    effective_listen: Sequence[tuple[str, int]]
    task_dispatcher: ThreadedTaskDispatcher
    def __init__(
        self,
        map: Incomplete | None = None,
        adj: Adjustments | None = None,
        effective_listen: Sequence[tuple[str, int]] | None = None,
        dispatcher: ThreadedTaskDispatcher | None = None,
    ) -> None: ...
    def print_listen(self, format_str: str) -> None: ...
    def run(self) -> None: ...
    def close(self) -> None: ...

class BaseWSGIServer(wasyncore.dispatcher):
    channel_class: HTTPChannel
    next_channel_cleanup: int
    socketmod: socket
    asyncore: Any
    sockinfo: tuple[int, int, int, tuple[str, int]]
    family: int
    socktype: int
    application: Any
    adj: Adjustments
    trigger: int
    task_dispatcher: ThreadedTaskDispatcher
    server_name: str
    active_channels: HTTPChannel
    def __init__(
        self,
        application: Any,
        map: Incomplete | None = None,
        _start: bool = True,
        _sock: Incomplete | None = None,
        dispatcher: ThreadedTaskDispatcher | None = None,
        adj: Adjustments | None = None,
        sockinfo: Incomplete | None = None,
        bind_socket: bool = True,
        **kw: Any,
    ) -> None: ...
    def bind_server_socket(self) -> None: ...
    def get_server_name(self, ip: str) -> str: ...
    def getsockname(self) -> Any: ...
    accepting: bool
    def accept_connections(self) -> None: ...
    def add_task(self, task: Task) -> None: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def handle_read(self) -> None: ...
    def handle_connect(self) -> None: ...
    def handle_accept(self) -> None: ...
    def run(self) -> None: ...
    def pull_trigger(self) -> None: ...
    def set_socket_options(self, conn: Any) -> None: ...
    def fix_addr(self, addr: Any) -> Any: ...
    def maintenance(self, now: int) -> None: ...
    def print_listen(self, format_str: str) -> None: ...
    def close(self) -> None: ...

class TcpWSGIServer(BaseWSGIServer):
    def bind_server_socket(self) -> None: ...
    def getsockname(self) -> tuple[str, tuple[str, int]]: ...
    def set_socket_options(self, conn: socket) -> None: ...

if sys.platform != "win32":
    class UnixWSGIServer(BaseWSGIServer):
        def __init__(
            self,
            application: Any,
            map: Incomplete | None = ...,
            _start: bool = ...,
            _sock: Incomplete | None = ...,
            dispatcher: Incomplete | None = ...,
            adj: Adjustments | None = ...,
            sockinfo: Incomplete | None = ...,
            **kw: Any,
        ) -> None: ...
        def bind_server_socket(self) -> None: ...
        def getsockname(self) -> tuple[str, tuple[str, int]]: ...
        def fix_addr(self, addr: Any) -> tuple[str, None]: ...
        def get_server_name(self, ip: Any) -> str: ...

WSGIServer: TcpWSGIServer



from _typeshed import Incomplete, StrOrBytesPath
from collections.abc import Sequence
from tarfile import TarInfo

def rebuild(
    filename: StrOrBytesPath,
    tag: Incomplete | None = None,
    format: str = "gz",
    zonegroups: Sequence[str | TarInfo] = [],
    metadata: Incomplete | None = None,
) -> None: ...



from _typeshed import Incomplete
from typing import Any

from .base import Client as Client

class WebApplicationClient(Client):
    grant_type: str
    code: Any
    def __init__(self, client_id, code: Incomplete | None = None, **kwargs) -> None: ...
    def prepare_request_uri(
        self,
        uri,
        redirect_uri: Incomplete | None = None,
        scope: Incomplete | None = None,
        state: Incomplete | None = None,
        code_challenge: str | None = None,
        code_challenge_method: str | None = "plain",
        **kwargs,
    ): ...
    def prepare_request_body(
        self,
        code: Incomplete | None = None,
        redirect_uri: Incomplete | None = None,
        body: str = "",
        include_client_id: bool = True,
        code_verifier: str | None = None,
        **kwargs,
    ): ...
    def parse_request_uri_response(self, uri, state: Incomplete | None = None): ...



from _typeshed import Incomplete

from influxdb_client.service._base_service import _BaseService

class ChecksService(_BaseService):
    def __init__(self, api_client: Incomplete | None = None) -> None: ...
    def create_check(self, post_check, **kwargs): ...
    def create_check_with_http_info(self, post_check, **kwargs): ...
    async def create_check_async(self, post_check, **kwargs): ...
    def delete_checks_id(self, check_id, **kwargs): ...
    def delete_checks_id_with_http_info(self, check_id, **kwargs): ...
    async def delete_checks_id_async(self, check_id, **kwargs): ...
    def delete_checks_id_labels_id(self, check_id, label_id, **kwargs): ...
    def delete_checks_id_labels_id_with_http_info(self, check_id, label_id, **kwargs): ...
    async def delete_checks_id_labels_id_async(self, check_id, label_id, **kwargs): ...
    def get_checks(self, org_id, **kwargs): ...
    def get_checks_with_http_info(self, org_id, **kwargs): ...
    async def get_checks_async(self, org_id, **kwargs): ...
    def get_checks_id(self, check_id, **kwargs): ...
    def get_checks_id_with_http_info(self, check_id, **kwargs): ...
    async def get_checks_id_async(self, check_id, **kwargs): ...
    def get_checks_id_labels(self, check_id, **kwargs): ...
    def get_checks_id_labels_with_http_info(self, check_id, **kwargs): ...
    async def get_checks_id_labels_async(self, check_id, **kwargs): ...
    def get_checks_id_query(self, check_id, **kwargs): ...
    def get_checks_id_query_with_http_info(self, check_id, **kwargs): ...
    async def get_checks_id_query_async(self, check_id, **kwargs): ...
    def patch_checks_id(self, check_id, check_patch, **kwargs): ...
    def patch_checks_id_with_http_info(self, check_id, check_patch, **kwargs): ...
    async def patch_checks_id_async(self, check_id, check_patch, **kwargs): ...
    def post_checks_id_labels(self, check_id, label_mapping, **kwargs): ...
    def post_checks_id_labels_with_http_info(self, check_id, label_mapping, **kwargs): ...
    async def post_checks_id_labels_async(self, check_id, label_mapping, **kwargs): ...
    def put_checks_id(self, check_id, check, **kwargs): ...
    def put_checks_id_with_http_info(self, check_id, check, **kwargs): ...
    async def put_checks_id_async(self, check_id, check, **kwargs): ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.utils.backends import _dispatch

__all__ = [
    "greedy_color",
    "strategy_connected_sequential",
    "strategy_connected_sequential_bfs",
    "strategy_connected_sequential_dfs",
    "strategy_independent_set",
    "strategy_largest_first",
    "strategy_random_sequential",
    "strategy_saturation_largest_first",
    "strategy_smallest_last",
]

@_dispatch
def strategy_largest_first(G, colors): ...
@_dispatch
def strategy_random_sequential(G, colors, seed: Incomplete | None = None): ...
@_dispatch
def strategy_smallest_last(G, colors): ...
@_dispatch
def strategy_independent_set(G, colors) -> Generator[Incomplete, Incomplete, None]: ...
@_dispatch
def strategy_connected_sequential_bfs(G, colors): ...
@_dispatch
def strategy_connected_sequential_dfs(G, colors): ...
@_dispatch
def strategy_connected_sequential(G, colors, traversal: str = "bfs") -> Generator[Incomplete, None, None]: ...
@_dispatch
def strategy_saturation_largest_first(G, colors) -> Generator[Incomplete, None, Incomplete]: ...
@_dispatch
def greedy_color(G, strategy: str = "largest_first", interchange: bool = False): ...

class _Node:
    node_id: Incomplete
    color: int
    adj_list: Incomplete
    adj_color: Incomplete
    def __init__(self, node_id, n) -> None: ...
    def assign_color(self, adj_entry, color) -> None: ...
    def clear_color(self, adj_entry, color) -> None: ...
    def iter_neighbors(self) -> Generator[Incomplete, None, None]: ...
    def iter_neighbors_color(self, color) -> Generator[Incomplete, None, None]: ...

class _AdjEntry:
    node_id: Incomplete
    next: Incomplete
    mate: Incomplete
    col_next: Incomplete
    col_prev: Incomplete
    def __init__(self, node_id) -> None: ...



from _typeshed import Incomplete

from influxdb_client.domain.expression import Expression

class MemberExpression(Expression):
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self, type: Incomplete | None = None, object: Incomplete | None = None, _property: Incomplete | None = None
    ) -> None: ...
    @property
    def type(self): ...
    @type.setter
    def type(self, type) -> None: ...
    @property
    def object(self): ...
    @object.setter
    def object(self, object) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import StrOrBytesPath
from collections.abc import Iterator
from io import IOBase
from typing import Any, Literal, final, overload
from typing_extensions import TypeAlias

from . import Index
from .enums import (
    ApplyLocation,
    BranchType,
    DiffFind,
    DiffFlag,
    DiffOption,
    DiffStatsFormat,
    FileMode,
    FileStatus,
    MergeAnalysis,
    MergePreference,
    ObjectType,
    Option,
    ReferenceFilter,
    ReferenceType,
    ResetMode,
    SortMode,
)
from .filter import Filter

GIT_APPLY_LOCATION_BOTH: int
GIT_APPLY_LOCATION_INDEX: int
GIT_APPLY_LOCATION_WORKDIR: int
GIT_BLAME_FIRST_PARENT: int
GIT_BLAME_IGNORE_WHITESPACE: int
GIT_BLAME_NORMAL: int
GIT_BLAME_TRACK_COPIES_ANY_COMMIT_COPIES: int
GIT_BLAME_TRACK_COPIES_SAME_COMMIT_COPIES: int
GIT_BLAME_TRACK_COPIES_SAME_COMMIT_MOVES: int
GIT_BLAME_TRACK_COPIES_SAME_FILE: int
GIT_BLAME_USE_MAILMAP: int
GIT_BLOB_FILTER_ATTRIBUTES_FROM_COMMIT: int
GIT_BLOB_FILTER_ATTRIBUTES_FROM_HEAD: int
GIT_BLOB_FILTER_CHECK_FOR_BINARY: int
GIT_BLOB_FILTER_NO_SYSTEM_ATTRIBUTES: int
GIT_BRANCH_ALL: int
GIT_BRANCH_LOCAL: int
GIT_BRANCH_REMOTE: int
GIT_CHECKOUT_ALLOW_CONFLICTS: int
GIT_CHECKOUT_CONFLICT_STYLE_DIFF3: int
GIT_CHECKOUT_CONFLICT_STYLE_MERGE: int
GIT_CHECKOUT_CONFLICT_STYLE_ZDIFF3: int
GIT_CHECKOUT_DISABLE_PATHSPEC_MATCH: int
GIT_CHECKOUT_DONT_OVERWRITE_IGNORED: int
GIT_CHECKOUT_DONT_REMOVE_EXISTING: int
GIT_CHECKOUT_DONT_UPDATE_INDEX: int
GIT_CHECKOUT_DONT_WRITE_INDEX: int
GIT_CHECKOUT_DRY_RUN: int
GIT_CHECKOUT_FORCE: int
GIT_CHECKOUT_NONE: int
GIT_CHECKOUT_NO_REFRESH: int
GIT_CHECKOUT_RECREATE_MISSING: int
GIT_CHECKOUT_REMOVE_IGNORED: int
GIT_CHECKOUT_REMOVE_UNTRACKED: int
GIT_CHECKOUT_SAFE: int
GIT_CHECKOUT_SKIP_LOCKED_DIRECTORIES: int
GIT_CHECKOUT_SKIP_UNMERGED: int
GIT_CHECKOUT_UPDATE_ONLY: int
GIT_CHECKOUT_USE_OURS: int
GIT_CHECKOUT_USE_THEIRS: int
GIT_CONFIG_HIGHEST_LEVEL: int
GIT_CONFIG_LEVEL_APP: int
GIT_CONFIG_LEVEL_GLOBAL: int
GIT_CONFIG_LEVEL_LOCAL: int
GIT_CONFIG_LEVEL_PROGRAMDATA: int
GIT_CONFIG_LEVEL_SYSTEM: int
GIT_CONFIG_LEVEL_XDG: int
GIT_DELTA_ADDED: int
GIT_DELTA_CONFLICTED: int
GIT_DELTA_COPIED: int
GIT_DELTA_DELETED: int
GIT_DELTA_IGNORED: int
GIT_DELTA_MODIFIED: int
GIT_DELTA_RENAMED: int
GIT_DELTA_TYPECHANGE: int
GIT_DELTA_UNMODIFIED: int
GIT_DELTA_UNREADABLE: int
GIT_DELTA_UNTRACKED: int
GIT_DESCRIBE_ALL: int
GIT_DESCRIBE_DEFAULT: int
GIT_DESCRIBE_TAGS: int
GIT_DIFF_BREAK_REWRITES: int
GIT_DIFF_BREAK_REWRITES_FOR_RENAMES_ONLY: int
GIT_DIFF_DISABLE_PATHSPEC_MATCH: int
GIT_DIFF_ENABLE_FAST_UNTRACKED_DIRS: int
GIT_DIFF_FIND_ALL: int
GIT_DIFF_FIND_AND_BREAK_REWRITES: int
GIT_DIFF_FIND_BY_CONFIG: int
GIT_DIFF_FIND_COPIES: int
GIT_DIFF_FIND_COPIES_FROM_UNMODIFIED: int
GIT_DIFF_FIND_DONT_IGNORE_WHITESPACE: int
GIT_DIFF_FIND_EXACT_MATCH_ONLY: int
GIT_DIFF_FIND_FOR_UNTRACKED: int
GIT_DIFF_FIND_IGNORE_LEADING_WHITESPACE: int
GIT_DIFF_FIND_IGNORE_WHITESPACE: int
GIT_DIFF_FIND_REMOVE_UNMODIFIED: int
GIT_DIFF_FIND_RENAMES: int
GIT_DIFF_FIND_RENAMES_FROM_REWRITES: int
GIT_DIFF_FIND_REWRITES: int
GIT_DIFF_FLAG_BINARY: int
GIT_DIFF_FLAG_EXISTS: int
GIT_DIFF_FLAG_NOT_BINARY: int
GIT_DIFF_FLAG_VALID_ID: int
GIT_DIFF_FLAG_VALID_SIZE: int
GIT_DIFF_FORCE_BINARY: int
GIT_DIFF_FORCE_TEXT: int
GIT_DIFF_IGNORE_BLANK_LINES: int
GIT_DIFF_IGNORE_CASE: int
GIT_DIFF_IGNORE_FILEMODE: int
GIT_DIFF_IGNORE_SUBMODULES: int
GIT_DIFF_IGNORE_WHITESPACE: int
GIT_DIFF_IGNORE_WHITESPACE_CHANGE: int
GIT_DIFF_IGNORE_WHITESPACE_EOL: int
GIT_DIFF_INCLUDE_CASECHANGE: int
GIT_DIFF_INCLUDE_IGNORED: int
GIT_DIFF_INCLUDE_TYPECHANGE: int
GIT_DIFF_INCLUDE_TYPECHANGE_TREES: int
GIT_DIFF_INCLUDE_UNMODIFIED: int
GIT_DIFF_INCLUDE_UNREADABLE: int
GIT_DIFF_INCLUDE_UNREADABLE_AS_UNTRACKED: int
GIT_DIFF_INCLUDE_UNTRACKED: int
GIT_DIFF_INDENT_HEURISTIC: int
GIT_DIFF_MINIMAL: int
GIT_DIFF_NORMAL: int
GIT_DIFF_PATIENCE: int
GIT_DIFF_RECURSE_IGNORED_DIRS: int
GIT_DIFF_RECURSE_UNTRACKED_DIRS: int
GIT_DIFF_REVERSE: int
GIT_DIFF_SHOW_BINARY: int
GIT_DIFF_SHOW_UNMODIFIED: int
GIT_DIFF_SHOW_UNTRACKED_CONTENT: int
GIT_DIFF_SKIP_BINARY_CHECK: int
GIT_DIFF_STATS_FULL: int
GIT_DIFF_STATS_INCLUDE_SUMMARY: int
GIT_DIFF_STATS_NONE: int
GIT_DIFF_STATS_NUMBER: int
GIT_DIFF_STATS_SHORT: int
GIT_DIFF_UPDATE_INDEX: int
GIT_FILEMODE_BLOB: int
GIT_FILEMODE_BLOB_EXECUTABLE: int
GIT_FILEMODE_COMMIT: int
GIT_FILEMODE_LINK: int
GIT_FILEMODE_TREE: int
GIT_FILEMODE_UNREADABLE: int
GIT_FILTER_ALLOW_UNSAFE: int
GIT_FILTER_ATTRIBUTES_FROM_COMMIT: int
GIT_FILTER_ATTRIBUTES_FROM_HEAD: int
GIT_FILTER_CLEAN: int
GIT_FILTER_DEFAULT: int
GIT_FILTER_DRIVER_PRIORITY: int
GIT_FILTER_NO_SYSTEM_ATTRIBUTES: int
GIT_FILTER_SMUDGE: int
GIT_FILTER_TO_ODB: int
GIT_FILTER_TO_WORKTREE: int
GIT_MERGE_ANALYSIS_FASTFORWARD: int
GIT_MERGE_ANALYSIS_NONE: int
GIT_MERGE_ANALYSIS_NORMAL: int
GIT_MERGE_ANALYSIS_UNBORN: int
GIT_MERGE_ANALYSIS_UP_TO_DATE: int
GIT_MERGE_PREFERENCE_FASTFORWARD_ONLY: int
GIT_MERGE_PREFERENCE_NONE: int
GIT_MERGE_PREFERENCE_NO_FASTFORWARD: int
GIT_OBJECT_ANY: int
GIT_OBJECT_BLOB: int
GIT_OBJECT_COMMIT: int
GIT_OBJECT_INVALID: int
GIT_OBJECT_OFS_DELTA: int
GIT_OBJECT_REF_DELTA: int
GIT_OBJECT_TAG: int
GIT_OBJECT_TREE: int
GIT_OBJ_ANY: int
GIT_OBJ_BLOB: int
GIT_OBJ_COMMIT: int
GIT_OBJ_TAG: int
GIT_OBJ_TREE: int
GIT_OID_HEXSZ: int
GIT_OID_HEX_ZERO: str
GIT_OID_MINPREFIXLEN: int
GIT_OID_RAWSZ: int
GIT_OPT_DISABLE_PACK_KEEP_FILE_CHECKS: int
GIT_OPT_ENABLE_CACHING: int
GIT_OPT_ENABLE_FSYNC_GITDIR: int
GIT_OPT_ENABLE_OFS_DELTA: int
GIT_OPT_ENABLE_STRICT_HASH_VERIFICATION: int
GIT_OPT_ENABLE_STRICT_OBJECT_CREATION: int
GIT_OPT_ENABLE_STRICT_SYMBOLIC_REF_CREATION: int
GIT_OPT_ENABLE_UNSAVED_INDEX_SAFETY: int
GIT_OPT_GET_CACHED_MEMORY: int
GIT_OPT_GET_MWINDOW_MAPPED_LIMIT: int
GIT_OPT_GET_MWINDOW_SIZE: int
GIT_OPT_GET_OWNER_VALIDATION: int
GIT_OPT_GET_PACK_MAX_OBJECTS: int
GIT_OPT_GET_SEARCH_PATH: int
GIT_OPT_GET_TEMPLATE_PATH: int
GIT_OPT_GET_USER_AGENT: int
GIT_OPT_GET_WINDOWS_SHAREMODE: int
GIT_OPT_SET_ALLOCATOR: int
GIT_OPT_SET_CACHE_MAX_SIZE: int
GIT_OPT_SET_CACHE_OBJECT_LIMIT: int
GIT_OPT_SET_MWINDOW_MAPPED_LIMIT: int
GIT_OPT_SET_MWINDOW_SIZE: int
GIT_OPT_SET_OWNER_VALIDATION: int
GIT_OPT_SET_PACK_MAX_OBJECTS: int
GIT_OPT_SET_SEARCH_PATH: int
GIT_OPT_SET_SSL_CERT_LOCATIONS: int
GIT_OPT_SET_SSL_CIPHERS: int
GIT_OPT_SET_TEMPLATE_PATH: int
GIT_OPT_SET_USER_AGENT: int
GIT_OPT_SET_WINDOWS_SHAREMODE: int
GIT_REFERENCES_ALL: int
GIT_REFERENCES_BRANCHES: int
GIT_REFERENCES_TAGS: int
GIT_REF_INVALID: int
GIT_REF_LISTALL: int
GIT_REF_OID: int
GIT_REF_SYMBOLIC: int
GIT_RESET_HARD: int
GIT_RESET_MIXED: int
GIT_RESET_SOFT: int
GIT_REVPARSE_MERGE_BASE: int
GIT_REVPARSE_RANGE: int
GIT_REVPARSE_SINGLE: int
GIT_REVSPEC_MERGE_BASE: int
GIT_REVSPEC_RANGE: int
GIT_REVSPEC_SINGLE: int
GIT_SORT_NONE: int
GIT_SORT_REVERSE: int
GIT_SORT_TIME: int
GIT_SORT_TOPOLOGICAL: int
GIT_STASH_APPLY_DEFAULT: int
GIT_STASH_APPLY_REINSTATE_INDEX: int
GIT_STASH_DEFAULT: int
GIT_STASH_INCLUDE_IGNORED: int
GIT_STASH_INCLUDE_UNTRACKED: int
GIT_STASH_KEEP_ALL: int
GIT_STASH_KEEP_INDEX: int
GIT_STATUS_CONFLICTED: int
GIT_STATUS_CURRENT: int
GIT_STATUS_IGNORED: int
GIT_STATUS_INDEX_DELETED: int
GIT_STATUS_INDEX_MODIFIED: int
GIT_STATUS_INDEX_NEW: int
GIT_STATUS_INDEX_RENAMED: int
GIT_STATUS_INDEX_TYPECHANGE: int
GIT_STATUS_WT_DELETED: int
GIT_STATUS_WT_MODIFIED: int
GIT_STATUS_WT_NEW: int
GIT_STATUS_WT_RENAMED: int
GIT_STATUS_WT_TYPECHANGE: int
GIT_STATUS_WT_UNREADABLE: int
GIT_SUBMODULE_IGNORE_ALL: int
GIT_SUBMODULE_IGNORE_DIRTY: int
GIT_SUBMODULE_IGNORE_NONE: int
GIT_SUBMODULE_IGNORE_UNSPECIFIED: int
GIT_SUBMODULE_IGNORE_UNTRACKED: int
GIT_SUBMODULE_STATUS_INDEX_ADDED: int
GIT_SUBMODULE_STATUS_INDEX_DELETED: int
GIT_SUBMODULE_STATUS_INDEX_MODIFIED: int
GIT_SUBMODULE_STATUS_IN_CONFIG: int
GIT_SUBMODULE_STATUS_IN_HEAD: int
GIT_SUBMODULE_STATUS_IN_INDEX: int
GIT_SUBMODULE_STATUS_IN_WD: int
GIT_SUBMODULE_STATUS_WD_ADDED: int
GIT_SUBMODULE_STATUS_WD_DELETED: int
GIT_SUBMODULE_STATUS_WD_INDEX_MODIFIED: int
GIT_SUBMODULE_STATUS_WD_MODIFIED: int
GIT_SUBMODULE_STATUS_WD_UNINITIALIZED: int
GIT_SUBMODULE_STATUS_WD_UNTRACKED: int
GIT_SUBMODULE_STATUS_WD_WD_MODIFIED: int
LIBGIT2_VERSION: str
LIBGIT2_VER_MAJOR: int
LIBGIT2_VER_MINOR: int
LIBGIT2_VER_REVISION: int

_GIT_OBJ_BLOB: TypeAlias = Literal[3]
_GIT_OBJ_COMMIT: TypeAlias = Literal[1]
_GIT_OBJ_TAG: TypeAlias = Literal[4]
_GIT_OBJ_TREE: TypeAlias = Literal[2]

class Object:
    _pointer: bytes
    filemode: FileMode
    hex: str
    id: Oid
    name: str | None
    oid: Oid
    raw_name: bytes | None
    short_id: str
    type: Literal[_GIT_OBJ_COMMIT, _GIT_OBJ_TREE, _GIT_OBJ_TAG, _GIT_OBJ_BLOB]
    type_str: Literal["commit", "tree", "tag", "blob"]
    @overload
    def peel(self, target_type: Literal[_GIT_OBJ_COMMIT]) -> Commit: ...
    @overload
    def peel(self, target_type: Literal[_GIT_OBJ_TREE]) -> Tree: ...
    @overload
    def peel(self, target_type: Literal[_GIT_OBJ_TAG]) -> Tag: ...
    @overload
    def peel(self, target_type: Literal[_GIT_OBJ_BLOB]) -> Blob: ...
    @overload
    def peel(self, target_type: None) -> Commit | Tree | Blob: ...
    def read_raw(self) -> bytes: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

@final
class Reference:
    name: str
    raw_name: bytes
    raw_shorthand: bytes
    raw_target: Oid | bytes
    shorthand: str
    target: Oid | str
    type: ReferenceType
    @overload
    def __init__(self, name: str, target: str) -> None: ...
    @overload
    def __init__(self, name: str, oid: Oid, peel: Oid) -> None: ...
    def delete(self) -> None: ...
    def log(self) -> Iterator[RefLogEntry]: ...
    @overload
    def peel(self, type: Literal[_GIT_OBJ_COMMIT]) -> Commit: ...
    @overload
    def peel(self, type: Literal[_GIT_OBJ_TREE]) -> Tree: ...
    @overload
    def peel(self, type: Literal[_GIT_OBJ_TAG]) -> Tag: ...
    @overload
    def peel(self, type: Literal[_GIT_OBJ_BLOB]) -> Blob: ...
    @overload
    def peel(self, type: None) -> Commit | Tree | Blob: ...
    def rename(self, new_name: str) -> None: ...
    def resolve(self) -> Reference: ...
    def set_target(self, target: _OidArg, message: str = ...) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __gt__(self, other: object) -> bool: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

class AlreadyExistsError(ValueError): ...

@final
class Blob(Object):
    data: bytes
    is_binary: bool
    size: int
    def diff(self, blob: Blob = ..., flag: int = ..., old_as_path: str = ..., new_as_path: str = ...) -> Patch: ...
    def diff_to_buffer(
        self, buffer: bytes | None = None, flag: DiffOption = ..., old_as_path: str = ..., buffer_as_path: str = ...
    ) -> Patch: ...

# This is not a real subclassing. Just ensuring type-checkers sees this type as compatible with _CDataBase
# pyright has no error code for subclassing final
@final
class Branch(Reference):  # type: ignore[misc]  # pyright: ignore
    branch_name: str
    raw_branch_name: bytes
    remote_name: str
    upstream: Branch
    upstream_name: str
    def delete(self) -> None: ...
    def is_checked_out(self) -> bool: ...
    def is_head(self) -> bool: ...
    def rename(self, name: str, force: bool = False) -> None: ...

@final
class Commit(Object):
    author: Signature
    commit_time: int
    commit_time_offset: int
    committer: Signature
    gpg_signature: tuple[bytes, bytes]
    message: str
    message_encoding: str
    message_trailers: dict[str, str]
    parent_ids: list[Oid]
    parents: list[Commit]
    raw_message: bytes
    tree: Tree
    tree_id: Oid

class Diff:
    deltas: Iterator[DiffDelta]
    patch: str | None
    patchid: Oid
    stats: DiffStats
    def find_similar(
        self,
        flags: DiffFind = ...,
        rename_threshold: int = 50,
        copy_threshold: int = 50,
        rename_from_rewrite_threshold: int = 50,
        break_rewrite_threshold: int = 60,
        rename_limit: int = 1000,
    ) -> None: ...
    def merge(self, diff: Diff) -> None: ...
    @staticmethod
    def from_c(diff: bytes, repo: Repository) -> Diff: ...
    @staticmethod
    def parse_diff(git_diff: str | bytes) -> Diff: ...
    def __getitem__(self, index: int) -> Patch: ...  # Diff_getitem
    def __iter__(self) -> Iterator[Patch]: ...  # -> DiffIter
    def __len__(self) -> int: ...

@final
class DiffDelta:
    flags: DiffFlag
    is_binary: bool
    nfiles: int
    new_file: DiffFile
    old_file: DiffFile
    similarity: int
    status: FileStatus
    def status_char(self) -> str: ...

@final
class DiffFile:
    flags: DiffFlag
    id: Oid
    mode: FileMode
    path: str
    raw_path: bytes
    size: int
    @staticmethod
    def from_c(ptr: bytes) -> DiffFile: ...

class DiffHunk:
    header: str
    lines: list[DiffLine]
    new_lines: int
    new_start: int
    old_lines: int
    old_start: int

@final
class DiffLine:
    content: str
    content_offset: int
    new_lineno: int
    num_lines: int
    old_lineno: int
    origin: str
    raw_content: bytes

class DiffStats:
    deletions: int
    files_changed: int
    insertions: int
    def format(self, format: DiffStatsFormat, width: int) -> str: ...

@final
class FilterSource:
    repo: Repository
    path: str
    filemode: int
    oid: Oid
    mode: int
    flags: int

class GitError(Exception): ...
class InvalidSpecError(ValueError): ...

@final
class Mailmap:
    def __init__(self) -> None: ...
    def add_entry(
        self, real_name: str = ..., real_email: str = ..., replace_name: str = ..., replace_email: str = ...
    ) -> None: ...
    @staticmethod
    def from_buffer(buffer: str | bytes) -> Mailmap: ...
    @staticmethod
    def from_repository(repository: Repository) -> Mailmap: ...
    def resolve(self, name: str, email: str) -> tuple[str, str]: ...
    def resolve_signature(self, sig: Signature) -> Signature: ...

@final
class Note:
    annotated_id: Oid
    id: Oid
    message: str
    def remove(self, author: Signature, committer: Signature, ref: str = "refs/notes/commits") -> None: ...

@final
class Odb:
    backends: Iterator[OdbBackend]
    def __init__(self, path: StrOrBytesPath | None = None) -> None: ...
    def add_backend(self, backend: OdbBackend, priority: int) -> None: ...
    def add_disk_alternate(self, path: str) -> None: ...
    def exists(self, oid: _OidArg) -> bool: ...
    def read(self, oid: _OidArg) -> tuple[int, int, bytes]: ...
    def write(self, type: int, data: bytes) -> Oid: ...
    def __contains__(self, other: _OidArg) -> bool: ...
    def __iter__(self) -> Iterator[Oid]: ...  # Odb_as_iter

class OdbBackend:
    def __init__(self) -> None: ...
    def exists(self, oid: _OidArg) -> bool: ...
    def exists_prefix(self, partial_id: _OidArg) -> Oid: ...
    def read(self, oid: _OidArg) -> tuple[int, bytes]: ...
    def read_header(self, oid: _OidArg) -> tuple[int, int]: ...
    def read_prefix(self, oid: _OidArg) -> tuple[int, bytes, Oid]: ...
    def refresh(self) -> None: ...
    def __iter__(self) -> Iterator[Oid]: ...  # OdbBackend_as_iter

@final
class OdbBackendLoose(OdbBackend):
    def __init__(
        self, objects_dir: StrOrBytesPath, compression_level: int, do_fsync: bool, dir_mode: int = 0, file_mode: int = 0
    ) -> None: ...

@final
class OdbBackendPack(OdbBackend):
    def __init__(self, path: StrOrBytesPath) -> None: ...

@final
class Oid:
    hex: str
    raw: bytes
    def __init__(self, raw: bytes = ..., hex: str = ...) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __gt__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

@final
class Patch:
    data: bytes
    delta: DiffDelta
    hunks: list[DiffHunk]
    line_stats: tuple[int, int, int]  # context, additions, deletions
    text: str | None

    @staticmethod
    def create_from(
        old: Blob | bytes | None,
        new: Blob | bytes | None,
        old_as_path: str = ...,
        new_as_path: str = ...,
        flag: DiffOption = ...,
        context_lines: int = 3,
        interhunk_lines: int = 0,
    ) -> Patch: ...

@final
class RefLogEntry:
    committer: Signature
    message: str
    oid_new: Oid
    oid_old: Oid

@final
class Refdb:
    def compress(self) -> None: ...
    @staticmethod
    def new(repo: Repository) -> Refdb: ...
    @staticmethod
    def open(repo: Repository) -> Refdb: ...
    def set_backend(self, backend: RefdbBackend) -> None: ...

class RefdbBackend:
    def __init__(self) -> None: ...
    def compress(self) -> None: ...
    def delete(self, ref_name: str, old_id: _OidArg, old_target: str) -> None: ...
    def ensure_log(self, ref_name: str) -> bool: ...
    def exists(self, refname: str) -> bool: ...
    def has_log(self, ref_name: str) -> bool: ...
    def lookup(self, refname: str) -> Reference: ...
    def rename(self, old_name: str, new_name: str, force: bool, who: Signature, message: str) -> Reference: ...
    def write(self, ref: Reference, force: bool, who: Signature, message: str, old: _OidArg, old_target: str) -> None: ...

@final
class RefdbFsBackend(RefdbBackend):
    def __init__(self, repo: Repository) -> None: ...

class Repository:
    _pointer: bytes
    default_signature: Signature
    head: Reference
    head_is_detached: bool
    head_is_unborn: bool
    is_bare: bool
    is_empty: bool
    is_shallow: bool
    odb: Odb
    path: str
    refdb: Refdb
    workdir: str
    def __init__(self, backend: object | None = None) -> None: ...
    def TreeBuilder(self, src: Tree | _OidArg = ...) -> TreeBuilder: ...
    def _disown(self) -> None: ...
    def _from_c(self, pointer: bytes, free: bool) -> None: ...
    def add_worktree(self, name: str, path: str, ref: Reference = ...) -> Worktree: ...
    def applies(self, diff: Diff, location: ApplyLocation = ..., raise_error: bool = False) -> bool: ...
    def apply(self, diff: Diff, location: ApplyLocation = ...) -> None: ...
    def cherrypick(self, id: _OidArg) -> None: ...
    def compress_references(self) -> None: ...
    def create_blob(self, data: bytes) -> Oid: ...
    def create_blob_fromdisk(self, path: str) -> Oid: ...
    def create_blob_fromiobase(self, iobase: IOBase) -> Oid: ...
    def create_blob_fromworkdir(self, path: str) -> Oid: ...
    def create_branch(self, name: str, commit: Commit, force: bool = False) -> Branch: ...
    def create_commit(
        self,
        reference_name: str | None,
        author: Signature,
        committer: Signature,
        message: str | bytes,
        tree: _OidArg,
        parents: list[_OidArg],
        encoding: str = ...,
    ) -> Oid: ...
    def create_commit_string(
        self,
        author: Signature,
        committer: Signature,
        message: str | bytes,
        tree: _OidArg,
        parents: list[_OidArg],
        encoding: str = ...,
    ) -> Oid: ...
    def create_commit_with_signature(self, content: str, signature: str, signature_field: str | None = None) -> Oid: ...
    def create_note(
        self,
        message: str,
        author: Signature,
        committer: Signature,
        annotated_id: str,
        ref: str = "refs/notes/commits",
        force: bool = False,
    ) -> Oid: ...
    def create_reference_direct(self, name: str, target: _OidArg, force: bool, message: str | None = None) -> Reference: ...
    def create_reference_symbolic(self, name: str, target: str, force: bool, message: str | None = None) -> Reference: ...
    def create_tag(self, name: str, oid: _OidArg, type: ObjectType, tagger: Signature, message: str) -> Oid: ...
    def descendant_of(self, oid1: _OidArg, oid2: _OidArg) -> bool: ...
    def expand_id(self, hex: str) -> Oid: ...
    def free(self) -> None: ...
    def git_object_lookup_prefix(self, oid: _OidArg) -> Object: ...
    def list_worktrees(self) -> list[str]: ...
    def listall_branches(self, flag: BranchType = ...) -> list[str]: ...
    def listall_mergeheads(self) -> list[Oid]: ...
    def listall_stashes(self) -> list[Stash]: ...
    def listall_submodules(self) -> list[str]: ...
    def lookup_branch(self, branch_name: str, branch_type: BranchType = ...) -> Branch: ...
    def lookup_note(self, annotated_id: str, ref: str = "refs/notes/commits") -> Note: ...
    def lookup_reference(self, name: str) -> Reference: ...
    def lookup_reference_dwim(self, name: str) -> Reference: ...
    def lookup_worktree(self, name: str) -> Worktree: ...
    def merge_analysis(self, their_head: _OidArg, our_ref: str = "HEAD") -> tuple[MergeAnalysis, MergePreference]: ...
    def merge_base(self, oid1: _OidArg, oid2: _OidArg) -> Oid: ...
    def merge_base_many(self, oids: list[_OidArg]) -> Oid: ...
    def merge_base_octopus(self, oids: list[_OidArg]) -> Oid: ...
    def notes(self) -> Iterator[Note]: ...
    def path_is_ignored(self, path: str) -> bool: ...
    def raw_listall_branches(self, flag: BranchType = ...) -> list[bytes]: ...
    def raw_listall_references(self) -> list[bytes]: ...
    def references_iterator_init(self) -> Iterator[Reference]: ...
    def references_iterator_next(self, iter: Iterator[Reference], references_return_type: ReferenceFilter = ...) -> Reference: ...
    def reset(self, oid: _OidArg, reset_type: ResetMode) -> None: ...
    def revparse(self, revspec: str) -> RevSpec: ...
    def revparse_ext(self, revision: str) -> tuple[Object, Reference]: ...
    def revparse_single(self, revision: str) -> Object: ...
    def set_odb(self, odb: Odb) -> None: ...
    def set_refdb(self, refdb: Refdb) -> None: ...
    def status(self, untracked_files: str = "all", ignored: bool = False) -> dict[str, int]: ...
    def status_file(self, path: str) -> int: ...
    def walk(self, oid: _OidArg | None, sort_mode: SortMode = ...) -> Walker: ...

class RevSpec:
    flags: int
    from_object: Object
    to_object: Object

@final
class Signature:
    _encoding: str | None
    _pointer: bytes
    email: str
    name: str
    offset: int
    raw_email: bytes
    raw_name: bytes
    time: int
    def __init__(self, name: str, email: str, time: int = -1, offset: int = 0, encoding: str | None = None) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __gt__(self, other: object) -> bool: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

@final
class Stash:
    commit_id: Oid
    message: str
    raw_message: bytes
    def __eq__(self, other: object) -> bool: ...
    def __ge__(self, other: object) -> bool: ...
    def __gt__(self, other: object) -> bool: ...
    def __le__(self, other: object) -> bool: ...
    def __lt__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

@final
class Tag(Object):
    message: str
    name: str
    raw_message: bytes
    raw_name: bytes
    tagger: Signature
    target: Oid
    def get_object(self) -> Object: ...

class Tree(Object):
    def diff_to_index(self, index: Index, flags: DiffOption = ..., context_lines: int = 3, interhunk_lines: int = 0) -> Diff: ...
    def diff_to_tree(
        self, tree: Tree = ..., flags: DiffOption = ..., context_lines: int = 3, interhunk_lines: int = 3, swap: bool = False
    ) -> Diff: ...
    def diff_to_workdir(self, flags: DiffOption = ..., context_lines: int = 3, interhunk_lines: int = 0) -> Diff: ...
    def __contains__(self, other: str) -> bool: ...  # Tree_contains
    def __getitem__(self, index: str | int) -> Object: ...  # Tree_subscript
    def __iter__(self) -> Iterator[Object]: ...
    def __len__(self) -> int: ...  # Tree_len
    def __rtruediv__(self, other: str) -> Object: ...
    def __truediv__(self, other: str) -> Object: ...  # Tree_divide

class TreeBuilder:
    def clear(self) -> None: ...
    def get(self, name: str) -> Object: ...
    def insert(self, name: str, oid: _OidArg, attr: int) -> None: ...
    def remove(self, name: str) -> None: ...
    def write(self) -> Oid: ...
    def __len__(self) -> int: ...

@final
class Walker:
    def hide(self, oid: _OidArg) -> None: ...
    def push(self, oid: _OidArg) -> None: ...
    def reset(self) -> None: ...
    def simplify_first_parent(self) -> None: ...
    def sort(self, mode: SortMode) -> None: ...
    def __iter__(self) -> Iterator[Commit]: ...  # Walker: ...
    def __next__(self) -> Commit: ...

@final
class Worktree:
    is_prunable: bool
    name: str
    path: str
    def prune(self, force: bool = False) -> None: ...

def discover_repository(path: str, across_fs: bool = False, ceiling_dirs: str = ...) -> str: ...
def filter_register(name: str, filter_cls: type[Filter], priority: int = ...) -> None: ...
def filter_unregister(name: str) -> None: ...
def hash(data: bytes) -> Oid: ...
def hashfile(path: str) -> Oid: ...
def init_file_backend(path: str, flags: int = 0) -> object: ...
def option(opt: Option, *args: Any) -> int | str | tuple[int, int] | None: ...
def reference_is_valid_name(refname: str) -> bool: ...
def tree_entry_cmp(a: Object, b: Object) -> int: ...
def _cache_enums() -> None: ...  # undocumented

_OidArg: TypeAlias = str | Oid



from _typeshed import Incomplete, SupportsRead, SupportsWrite, Unused
from collections.abc import Callable, Iterable, Iterator, MutableMapping, Sequence
from enum import IntEnum
from pathlib import Path
from typing import Any, ClassVar, Literal, Protocol, SupportsBytes
from typing_extensions import Self, TypeAlias, TypeGuard

from PIL.PyAccess import PyAccess

from ._imaging import (
    DEFAULT_STRATEGY as DEFAULT_STRATEGY,
    FILTERED as FILTERED,
    FIXED as FIXED,
    HUFFMAN_ONLY as HUFFMAN_ONLY,
    RLE as RLE,
    _PixelAccessor,
)
from .ImageFilter import Filter
from .ImagePalette import ImagePalette

_Mode: TypeAlias = str
_Resample: TypeAlias = Literal[0, 1, 2, 3, 4, 5]
_Size: TypeAlias = tuple[int, int]
_Box: TypeAlias = tuple[int, int, int, int]

_ConversionMatrix: TypeAlias = (
    tuple[float, float, float, float] | tuple[float, float, float, float, float, float, float, float, float, float, float, float]
)
# `str` values are only accepted if mode="RGB" for an `Image` object
# `float` values are only accepted for certain modes such as "F"
# See https://pillow.readthedocs.io/en/stable/reference/Image.html#PIL.Image.new
_Color: TypeAlias = (
    int | tuple[int] | tuple[int, int] | tuple[int, int, int] | tuple[int, int, int, int] | str | float | tuple[float]
)

class _Writeable(SupportsWrite[bytes], Protocol):
    def seek(self, offset: int, /) -> Any: ...

# Ref: https://numpy.org/doc/stable/reference/arrays.interface.html#python-side
class _SupportsArrayInterface(Protocol):
    @property
    def __array_interface__(self) -> dict[str, Any]: ...

class DecompressionBombWarning(RuntimeWarning): ...
class DecompressionBombError(Exception): ...

# Despite the ALL_CAPS spelling, Pillow's docs mention that this threshold can
# be altered at runtime. See
#     https://pillow.readthedocs.io/en/stable/reference/Image.html#PIL.Image.open
# or the permalink
#     https://github.com/python-pillow/Pillow/blob/10.0.0/docs/reference/Image.rst?plain=1#L54-L55
MAX_IMAGE_PIXELS: int | None

USE_CFFI_ACCESS: bool

def isImageType(t: object) -> TypeGuard[Image]: ...

class Transpose(IntEnum):
    FLIP_LEFT_RIGHT: Literal[0]
    FLIP_TOP_BOTTOM: Literal[1]
    ROTATE_90: Literal[2]
    ROTATE_180: Literal[3]
    ROTATE_270: Literal[4]
    TRANSPOSE: Literal[5]
    TRANSVERSE: Literal[6]

# All Transpose items
FLIP_LEFT_RIGHT: Literal[0]
FLIP_TOP_BOTTOM: Literal[1]
ROTATE_90: Literal[2]
ROTATE_180: Literal[3]
ROTATE_270: Literal[4]
TRANSPOSE: Literal[5]
TRANSVERSE: Literal[6]

class Transform(IntEnum):
    AFFINE: Literal[0]
    EXTENT: Literal[1]
    PERSPECTIVE: Literal[2]
    QUAD: Literal[3]
    MESH: Literal[4]

# All Transform items
AFFINE: Literal[0]
EXTENT: Literal[1]
PERSPECTIVE: Literal[2]
QUAD: Literal[3]
MESH: Literal[4]

class Resampling(IntEnum):
    NEAREST: Literal[0]
    LANCZOS: Literal[1]
    BILINEAR: Literal[2]
    BICUBIC: Literal[3]
    BOX: Literal[4]
    HAMMING: Literal[5]

# All Resampling items
NEAREST: Literal[0]
LANCZOS: Literal[1]
BILINEAR: Literal[2]
BICUBIC: Literal[3]
BOX: Literal[4]
HAMMING: Literal[5]

class Dither(IntEnum):
    NONE: Literal[0]
    ORDERED: Literal[1]
    RASTERIZE: Literal[2]
    FLOYDSTEINBERG: Literal[3]

# All Dither items
NONE: Literal[0]
ORDERED: Literal[1]
RASTERIZE: Literal[2]
FLOYDSTEINBERG: Literal[3]

class Palette(IntEnum):
    WEB: Literal[0]
    ADAPTIVE: Literal[1]

# All Palette items
WEB: Literal[0]
ADAPTIVE: Literal[1]

class Quantize(IntEnum):
    MEDIANCUT: Literal[0]
    MAXCOVERAGE: Literal[1]
    FASTOCTREE: Literal[2]
    LIBIMAGEQUANT: Literal[3]

# All Quantize items
MEDIANCUT: Literal[0]
MAXCOVERAGE: Literal[1]
FASTOCTREE: Literal[2]
LIBIMAGEQUANT: Literal[3]

ID: list[str]
OPEN: dict[str, Any]
MIME: dict[str, str]
SAVE: dict[str, Any]
SAVE_ALL: dict[str, Any]
EXTENSION: dict[str, str]
DECODERS: dict[str, Any]
ENCODERS: dict[str, Any]

MODES: list[_Mode]

def getmodebase(mode: _Mode) -> Literal["L", "RGB"]: ...
def getmodetype(mode: _Mode) -> Literal["L", "I", "F"]: ...
def getmodebandnames(mode: _Mode) -> tuple[str, ...]: ...
def getmodebands(mode: _Mode) -> int: ...
def preinit() -> None: ...
def init() -> None: ...

class _E:
    scale: Incomplete
    offset: Incomplete
    def __init__(self, scale, offset) -> None: ...
    def __neg__(self) -> _E: ...
    def __add__(self, other) -> _E: ...
    __radd__ = __add__
    def __sub__(self, other): ...
    def __rsub__(self, other): ...
    def __mul__(self, other) -> _E: ...
    __rmul__ = __mul__
    def __truediv__(self, other) -> _E: ...

_ImageState: TypeAlias = tuple[dict[str, Any], str, tuple[int, int], Any, bytes]

class Image:
    format: ClassVar[str | None]
    format_description: ClassVar[str | None]
    im: Incomplete
    palette: Incomplete
    info: dict[Incomplete, Incomplete]
    readonly: int
    pyaccess: PyAccess | None
    is_animated: bool  # not present on all Image objects
    n_frames: int  # not present on all Image objects
    # Only defined after a call to save().
    encoderconfig: tuple[Incomplete, ...]
    @property
    def width(self) -> int: ...
    @property
    def height(self) -> int: ...
    @property
    def size(self) -> tuple[int, int]: ...
    @property
    def mode(self) -> _Mode: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: Unused) -> None: ...
    def close(self) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    @property
    def __array_interface__(self): ...
    def __getstate__(self) -> _ImageState: ...
    def __setstate__(self, state: _ImageState) -> None: ...
    def tobytes(self, encoder_name: str = "raw", *args) -> bytes: ...
    def tobitmap(self, name: str = "image") -> bytes: ...
    def frombytes(self, data: bytes, decoder_name: str = "raw", *args) -> None: ...
    def load(self) -> _PixelAccessor: ...
    def verify(self) -> None: ...
    def convert(
        self,
        mode: _Mode | None = None,
        matrix: _ConversionMatrix | None = None,
        dither: int | None = None,
        palette: Palette | Literal[0, 1] = ...,
        colors: int = 256,
    ) -> Image: ...
    def quantize(
        self,
        colors: int = 256,
        method: Quantize | Literal[0, 1, 2, 3] | None = None,
        kmeans: int = 0,
        palette: Image | None = None,
        dither: int = ...,
    ) -> Image: ...
    def copy(self) -> Image: ...
    __copy__ = copy
    def crop(self, box: _Box | None = None) -> Image: ...
    def draft(self, mode: _Mode, size: _Size) -> None: ...
    def filter(self, filter: Filter | Callable[[], Filter]) -> Image: ...
    def getbands(self) -> tuple[str, ...]: ...
    def getbbox(self, *, alpha_only: bool = True) -> tuple[int, int, int, int] | None: ...
    def getcolors(self, maxcolors: int = 256) -> list[tuple[int, int]]: ...
    def getdata(self, band: int | None = None): ...
    def getextrema(self): ...
    def getexif(self) -> Exif: ...
    def get_child_images(self) -> list[Image]: ...
    def getim(self): ...
    def getpalette(self, rawmode: str | None = "RGB") -> list[int] | None: ...
    @property
    def has_transparency_data(self) -> bool: ...
    def apply_transparency(self) -> None: ...
    def getpixel(self, xy: tuple[int, int]): ...
    def getprojection(self) -> tuple[list[int], list[int]]: ...
    def histogram(self, mask: Image | None = None, extrema: tuple[int, int] | tuple[float, float] | None = None) -> list[int]: ...
    def entropy(self, mask: Image | None = None, extrema: tuple[int, int] | tuple[float, float] | None = None) -> float: ...
    def paste(self, im: Image | _Color, box: tuple[int, int] | _Box | None = None, mask: Image | None = None) -> None: ...
    def alpha_composite(self, im: Image, dest: tuple[int, int] = (0, 0), source: tuple[int, int] = (0, 0)) -> None: ...
    def point(self, lut, mode: _Mode | None = None) -> Image: ...
    def putalpha(self, alpha: Image | int) -> None: ...
    def putdata(self, data: Sequence[int], scale: float = 1.0, offset: float = 0.0) -> None: ...
    def putpalette(self, data: ImagePalette | bytes | Iterable[int] | SupportsBytes, rawmode: _Mode | None = "RGB") -> None: ...
    def putpixel(self, xy: tuple[int, int], value: _Color | list[float]) -> None: ...
    def remap_palette(self, dest_map: Iterable[int], source_palette: Sequence[int] | None = None) -> Image: ...
    def resize(
        self,
        size: tuple[int, int],
        resample: Resampling | _Resample | None = None,
        box: tuple[float, float, float, float] | None = None,
        reducing_gap: float | None = None,
    ) -> Image: ...
    def reduce(self, factor: int | tuple[int, int] | list[int], box: _Box | None = None) -> Image: ...
    def rotate(
        self,
        angle: float,
        resample: Resampling | _Resample = ...,
        expand: bool = ...,
        center: tuple[float, float] | None = None,
        translate: tuple[float, float] | None = None,
        fillcolor: _Color | None = None,
    ) -> Image: ...
    def save(
        self,
        fp: str | bytes | Path | _Writeable,
        format: str | None = None,
        *,
        save_all: bool = ...,
        bitmap_format: Literal["bmp", "png"] = ...,  # for ICO files
        optimize: bool = ...,
        **params: Any,
    ) -> None: ...
    def seek(self, frame: int) -> None: ...
    def show(self, title: str | None = None) -> None: ...
    def split(self) -> tuple[Image, ...]: ...
    def getchannel(self, channel: int | str) -> Image: ...
    def tell(self) -> int: ...
    def thumbnail(self, size: tuple[int, int], resample: Resampling | _Resample = ..., reducing_gap: float = 2.0) -> None: ...
    def transform(
        self,
        size: _Size,
        method: Transform | Literal[0, 1, 2, 3, 4],
        data=None,
        resample: Resampling | _Resample = ...,
        fill: int = 1,
        fillcolor: _Color | int | None = None,
    ) -> Image: ...
    def transpose(self, method: Transpose | Literal[0, 1, 2, 3, 4, 5, 6]) -> Image: ...
    def effect_spread(self, distance: int) -> Image: ...
    def toqimage(self): ...
    def toqpixmap(self): ...

class ImagePointHandler: ...
class ImageTransformHandler: ...

def new(mode: _Mode, size: tuple[int, int], color: _Color = 0) -> Image: ...
def frombytes(mode: _Mode, size: tuple[int, int], data, decoder_name: str = "raw", *args) -> Image: ...
def frombuffer(mode: _Mode, size: tuple[int, int], data, decoder_name: str = "raw", *args) -> Image: ...

# If the __array_interface__ has "strides", then `obj` must also support `tobytes` or `tostring`, but we can't enforce that
def fromarray(obj: _SupportsArrayInterface, mode: _Mode | None = None) -> Image: ...
def fromqimage(im) -> Image: ...
def fromqpixmap(im) -> Image: ...
def open(
    fp: str | bytes | Path | SupportsRead[bytes], mode: Literal["r"] = "r", formats: list[str] | tuple[str, ...] | None = None
) -> Image: ...
def alpha_composite(im1: Image, im2: Image) -> Image: ...
def blend(im1: Image, im2: Image, alpha: float) -> Image: ...
def composite(image1: Image, image2: Image, mask: Image) -> Image: ...
def eval(image: Image, *args) -> Image: ...
def merge(mode: _Mode, bands: Sequence[Image]) -> Image: ...
def register_open(id: str, factory, accept=None) -> None: ...
def register_mime(id: str, mimetype: str) -> None: ...
def register_save(id: str, driver) -> None: ...
def register_save_all(id: str, driver) -> None: ...
def register_extension(id: str, extension: str) -> None: ...
def register_extensions(id: str, extensions: Iterable[str]) -> None: ...
def registered_extensions() -> dict[str, str]: ...
def register_decoder(name: str, decoder) -> None: ...
def register_encoder(name: str, encoder) -> None: ...
def effect_mandelbrot(size: tuple[int, int], extent: tuple[float, float, float, float], quality: int) -> Image: ...
def effect_noise(size: tuple[int, int], sigma: float) -> Image: ...
def linear_gradient(mode: _Mode) -> Image: ...
def radial_gradient(mode: _Mode) -> Image: ...

class Exif(MutableMapping[int, Any]):
    endian: Incomplete
    bigtiff: bool
    def load(self, data: bytes) -> None: ...
    def load_from_fp(self, fp, offset: Incomplete | None = None) -> None: ...
    def tobytes(self, offset: int = 8) -> bytes: ...
    def get_ifd(self, tag: int): ...
    def hide_offsets(self) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, tag: int) -> Any: ...
    def __contains__(self, tag: object) -> bool: ...
    def __setitem__(self, tag: int, value: Any) -> None: ...
    def __delitem__(self, tag: int) -> None: ...
    def __iter__(self) -> Iterator[int]: ...



from _typeshed import Incomplete
from collections.abc import Callable, Mapping, Sequence
from typing import Any

from consolemenu.console_menu import ConsoleMenu
from consolemenu.items import ExternalItem as ExternalItem

class FunctionItem(ExternalItem):
    function: Callable[..., Any]
    args: Sequence[Any]
    kwargs: Mapping[str, Any]
    return_value: Incomplete | None
    def __init__(
        self,
        text: str,
        function: Callable[..., Any],
        args: Sequence[Any] | None = None,
        kwargs: Mapping[str, Any] | None = None,
        menu: ConsoleMenu | None = None,
        should_exit: bool = False,
        menu_char: str | None = None,
    ) -> None: ...
    def action(self) -> None: ...
    def clean_up(self) -> None: ...
    def get_return(self) -> Any | None: ...



from _typeshed import Incomplete

class Permission:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(self, action: Incomplete | None = None, resource: Incomplete | None = None) -> None: ...
    @property
    def action(self): ...
    @action.setter
    def action(self, action) -> None: ...
    @property
    def resource(self): ...
    @resource.setter
    def resource(self, resource) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete

class RetentionPolicyManifest:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        name: Incomplete | None = None,
        replica_n: Incomplete | None = None,
        duration: Incomplete | None = None,
        shard_group_duration: Incomplete | None = None,
        shard_groups: Incomplete | None = None,
        subscriptions: Incomplete | None = None,
    ) -> None: ...
    @property
    def name(self): ...
    @name.setter
    def name(self, name) -> None: ...
    @property
    def replica_n(self): ...
    @replica_n.setter
    def replica_n(self, replica_n) -> None: ...
    @property
    def duration(self): ...
    @duration.setter
    def duration(self, duration) -> None: ...
    @property
    def shard_group_duration(self): ...
    @shard_group_duration.setter
    def shard_group_duration(self, shard_group_duration) -> None: ...
    @property
    def shard_groups(self): ...
    @shard_groups.setter
    def shard_groups(self, shard_groups) -> None: ...
    @property
    def subscriptions(self): ...
    @subscriptions.setter
    def subscriptions(self, subscriptions) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete

class BuilderTagsType:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self, key: Incomplete | None = None, values: Incomplete | None = None, aggregate_function_type: Incomplete | None = None
    ) -> None: ...
    @property
    def key(self): ...
    @key.setter
    def key(self, key) -> None: ...
    @property
    def values(self): ...
    @values.setter
    def values(self, values) -> None: ...
    @property
    def aggregate_function_type(self): ...
    @aggregate_function_type.setter
    def aggregate_function_type(self, aggregate_function_type) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



import _csv
from _typeshed import FileDescriptorOrPath, StrOrBytesPath
from collections.abc import Iterable
from typing import Any, BinaryIO, TextIO
from typing_extensions import TypeAlias

from netaddr.core import Publisher, Subscriber

_INDEX: TypeAlias = dict[int, list[tuple[int, int]]]
OUI_INDEX: _INDEX
IAB_INDEX: _INDEX

class FileIndexer(Subscriber):
    writer: _csv._writer
    def __init__(self, index_file: TextIO | FileDescriptorOrPath) -> None: ...
    def update(self, data: Iterable[Any]) -> None: ...

class OUIIndexParser(Publisher):
    fh: BinaryIO
    def __init__(self, ieee_file: BinaryIO | FileDescriptorOrPath) -> None: ...
    def parse(self) -> None: ...

class IABIndexParser(Publisher):
    fh: BinaryIO
    def __init__(self, ieee_file: BinaryIO | FileDescriptorOrPath) -> None: ...
    def parse(self) -> None: ...

def create_index_from_registry(
    registry_fh: BinaryIO | FileDescriptorOrPath, index_path: StrOrBytesPath, parser: type[OUIIndexParser | IABIndexParser]
) -> None: ...
def create_indices() -> None: ...
def load_index(index: _INDEX, fp: Iterable[bytes]) -> None: ...
def load_indices() -> None: ...



from _typeshed import Incomplete
from collections.abc import Hashable, Mapping
from typing import Literal, TypeVar, overload

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph
from networkx.classes.multidigraph import MultiDiGraph
from networkx.classes.multigraph import MultiGraph
from networkx.utils.backends import _dispatch

_X = TypeVar("_X", bound=Hashable)
_Y = TypeVar("_Y", bound=Hashable)

@overload
def relabel_nodes(G: MultiDiGraph[_X], mapping: Mapping[_X, _Y], copy: bool = True) -> MultiDiGraph[_X | _Y]: ...
@overload
def relabel_nodes(G: DiGraph[_X], mapping: Mapping[_X, _Y], copy: bool = True) -> DiGraph[_X | _Y]: ...
@overload
def relabel_nodes(G: MultiGraph[_X], mapping: Mapping[_X, _Y], copy: bool = True) -> MultiGraph[_X | _Y]: ...
@overload
def relabel_nodes(G: Graph[_X], mapping: Mapping[_X, _Y], copy: bool = True) -> Graph[_X | _Y]: ...
@_dispatch
def convert_node_labels_to_integers(
    G: Graph[Hashable],
    first_label: int = 0,
    ordering: Literal["default", "sorted", "increasing degree", "decreasing degree"] = "default",
    label_attribute: Incomplete | None = None,
) -> Graph[int]: ...



from collections.abc import Sequence
from typing import Any

class JMESPathError(ValueError): ...

class ParseError(JMESPathError):
    lex_position: int
    token_value: str
    token_type: str
    msg: str
    expression: str | None
    def __init__(
        self, lex_position: int, token_value: str, token_type: str, msg: str = "Invalid jmespath expression"
    ) -> None: ...

class IncompleteExpressionError(ParseError):
    # When ParseError is used directly, the token always have a non-null value and type
    token_value: str | None  # type: ignore[assignment]
    token_type: str | None  # type: ignore[assignment]
    expression: str
    def set_expression(self, expression: str) -> None: ...

class LexerError(ParseError):
    lexer_position: int
    lexer_value: str
    message: str
    def __init__(self, lexer_position: int, lexer_value: str, message: str, expression: str | None = None) -> None: ...

class ArityError(ParseError):
    expected_arity: int
    actual_arity: int
    function_name: str
    def __init__(self, expected, actual, name) -> None: ...

class VariadictArityError(ArityError): ...

class JMESPathTypeError(JMESPathError):
    function_name: str
    current_value: Any
    actual_type: str
    expected_types: Sequence[str]
    def __init__(self, function_name: str, current_value: Any, actual_type: str, expected_types: Sequence[str]) -> None: ...

class EmptyExpressionError(JMESPathError):
    def __init__(self) -> None: ...

class UnknownFunctionError(JMESPathError): ...



from _typeshed import Incomplete

from networkx.algorithms.flow import edmonds_karp
from networkx.utils.backends import _dispatch

__all__ = ["k_components"]

default_flow_func = edmonds_karp

@_dispatch
def k_components(G, flow_func: Incomplete | None = None): ...



from _typeshed import Incomplete, SupportsWrite
from abc import ABC, abstractmethod
from collections.abc import Iterable, Mapping
from typing import NoReturn, TypeVar, overload

from .std import tqdm as std_tqdm

__all__ = ["tqdm_rich", "trrange", "tqdm", "trange"]

_T = TypeVar("_T")

# Actually rich.progress.ProgressColumn
class _ProgressColumn(ABC):
    max_refresh: float | None
    def __init__(self, table_column: Incomplete | None = ...) -> None: ...
    def get_table_column(self) -> Incomplete: ...
    def __call__(self, task: Incomplete) -> Incomplete: ...
    @abstractmethod
    def render(self, task: Incomplete) -> Incomplete: ...

class FractionColumn(_ProgressColumn):
    unit_scale: bool
    unit_divisor: int

    def __init__(self, unit_scale: bool = ..., unit_divisor: int = ...) -> None: ...
    def render(self, task): ...

class RateColumn(_ProgressColumn):
    unit: str
    unit_scale: bool
    unit_divisor: int

    def __init__(self, unit: str = ..., unit_scale: bool = ..., unit_divisor: int = ...) -> None: ...
    def render(self, task): ...

class tqdm_rich(std_tqdm[_T]):
    def close(self) -> None: ...
    def clear(self, *_, **__) -> None: ...
    def display(self, *_, **__) -> None: ...
    def reset(self, total: Incomplete | None = ...) -> None: ...
    @overload
    def __init__(
        self,
        iterable: Iterable[_T],
        desc: str | None = ...,
        total: float | None = ...,
        leave: bool | None = ...,
        file: SupportsWrite[str] | None = ...,
        ncols: int | None = ...,
        mininterval: float = ...,
        maxinterval: float = ...,
        miniters: float | None = ...,
        ascii: bool | str | None = ...,
        disable: bool | None = ...,
        unit: str = ...,
        unit_scale: bool | float = ...,
        dynamic_ncols: bool = ...,
        smoothing: float = ...,
        bar_format: str | None = ...,
        initial: float = ...,
        position: int | None = ...,
        postfix: Mapping[str, object] | str | None = ...,
        unit_divisor: float = ...,
        write_bytes: bool | None = ...,
        lock_args: tuple[bool | None, float | None] | tuple[bool | None] | None = ...,
        nrows: int | None = ...,
        colour: str | None = ...,
        delay: float | None = ...,
        gui: bool = ...,
        **kwargs,
    ) -> None: ...
    @overload
    def __init__(
        self: tqdm_rich[NoReturn],
        iterable: None = None,
        desc: str | None = ...,
        total: float | None = ...,
        leave: bool | None = ...,
        file: SupportsWrite[str] | None = ...,
        ncols: int | None = ...,
        mininterval: float = ...,
        maxinterval: float = ...,
        miniters: float | None = ...,
        ascii: bool | str | None = ...,
        disable: bool | None = ...,
        unit: str = ...,
        unit_scale: bool | float = ...,
        dynamic_ncols: bool = ...,
        smoothing: float = ...,
        bar_format: str | None = ...,
        initial: float = ...,
        position: int | None = ...,
        postfix: Mapping[str, object] | str | None = ...,
        unit_divisor: float = ...,
        write_bytes: bool | None = ...,
        lock_args: tuple[bool | None, float | None] | tuple[bool | None] | None = ...,
        nrows: int | None = ...,
        colour: str | None = ...,
        delay: float | None = ...,
        gui: bool = ...,
        **kwargs,
    ) -> None: ...

def trrange(*args, **kwargs) -> tqdm_rich[int]: ...

tqdm = tqdm_rich
trange = trrange



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def complete_bipartite_graph(n1, n2, create_using: Incomplete | None = None): ...
@_dispatch
def configuration_model(aseq, bseq, create_using: Incomplete | None = None, seed: Incomplete | None = None): ...
@_dispatch
def havel_hakimi_graph(aseq, bseq, create_using: Incomplete | None = None): ...
@_dispatch
def reverse_havel_hakimi_graph(aseq, bseq, create_using: Incomplete | None = None): ...
@_dispatch
def alternating_havel_hakimi_graph(aseq, bseq, create_using: Incomplete | None = None): ...
@_dispatch
def preferential_attachment_graph(aseq, p, create_using: Incomplete | None = None, seed: Incomplete | None = None): ...
@_dispatch
def random_graph(n, m, p, seed: Incomplete | None = None, directed: bool = False): ...
@_dispatch
def gnmk_random_graph(n, m, k, seed: Incomplete | None = None, directed: bool = False): ...



from typing import Final

Approve: Final = "approve_me"
InsufficientFundsContactUs: Final = "insufficient_funds__contact"
AccountNotAuthorizedContactUs: Final = "account_not_authorized__contact"
BankRejectedUpdateFundingInformation: Final = "bank_rejected__update"
BankRejectedNone: Final = "bank_rejected__none"



from .backend_application import BackendApplicationClient as BackendApplicationClient
from .base import AUTH_HEADER as AUTH_HEADER, BODY as BODY, URI_QUERY as URI_QUERY, Client as Client
from .legacy_application import LegacyApplicationClient as LegacyApplicationClient
from .mobile_application import MobileApplicationClient as MobileApplicationClient
from .service_application import ServiceApplicationClient as ServiceApplicationClient
from .web_application import WebApplicationClient as WebApplicationClient



from _typeshed import Incomplete
from collections.abc import Iterator
from dataclasses import dataclass
from enum import Enum

from networkx.utils.backends import _dispatch

class EdgePartition(Enum):
    OPEN = 0
    INCLUDED = 1
    EXCLUDED = 2

@_dispatch
def minimum_spanning_edges(
    G, algorithm: str = "kruskal", weight: str = "weight", keys: bool = True, data: bool = True, ignore_nan: bool = False
): ...
@_dispatch
def maximum_spanning_edges(
    G, algorithm: str = "kruskal", weight: str = "weight", keys: bool = True, data: bool = True, ignore_nan: bool = False
): ...
@_dispatch
def minimum_spanning_tree(G, weight: str = "weight", algorithm: str = "kruskal", ignore_nan: bool = False): ...
@_dispatch
def partition_spanning_tree(
    G, minimum: bool = True, weight: str = "weight", partition: str = "partition", ignore_nan: bool = False
): ...
@_dispatch
def maximum_spanning_tree(G, weight: str = "weight", algorithm: str = "kruskal", ignore_nan: bool = False): ...
@_dispatch
def random_spanning_tree(G, weight: Incomplete | None = None, *, multiplicative: bool = True, seed: Incomplete | None = None): ...

class SpanningTreeIterator:
    @dataclass
    class Partition:
        mst_weight: float
        partition_dict: dict[Incomplete, Incomplete]

    G: Incomplete
    weight: Incomplete
    minimum: Incomplete
    ignore_nan: Incomplete
    partition_key: str
    def __init__(self, G, weight: str = "weight", minimum: bool = True, ignore_nan: bool = False) -> None: ...
    partition_queue: Incomplete
    def __iter__(self) -> Iterator[Incomplete]: ...
    def __next__(self): ...



from _typeshed import Incomplete
from collections.abc import Generator
from typing import ClassVar

from ..core import OrthodoxCalendar

class Bulgaria(OrthodoxCalendar):
    FIXED_HOLIDAYS: Incomplete
    include_labour_day: ClassVar[bool]
    labour_day_label: ClassVar[str]
    include_good_friday: ClassVar[bool]
    include_easter_saturday: ClassVar[bool]
    include_easter_sunday: ClassVar[bool]
    include_easter_monday: ClassVar[bool]
    include_christmas_eve: ClassVar[bool]
    include_christmas: ClassVar[bool]
    include_boxing_day: ClassVar[bool]
    include_orthodox_christmas: ClassVar[bool]
    boxing_day_label: ClassVar[str]

    def get_shifted_holidays(self, days) -> Generator[Incomplete, None, None]: ...
    def get_fixed_holidays(self, year): ...
    def shift_christmas_boxing_days(self, year): ...
    def get_variable_days(self, year): ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import WesternCalendar

class Iceland(WesternCalendar):
    include_holy_thursday: ClassVar[bool]
    include_good_friday: ClassVar[bool]
    include_easter_monday: ClassVar[bool]
    include_ascension: ClassVar[bool]
    include_whit_monday: ClassVar[bool]
    include_christmas_eve: ClassVar[bool]
    include_boxing_day: ClassVar[bool]
    boxing_day_label: ClassVar[str]
    include_labour_day: ClassVar[bool]
    FIXED_HOLIDAYS: Incomplete
    def get_first_day_of_summer(self, year): ...
    def get_commerce_day(self, year): ...
    def get_variable_days(self, year): ...



from typing import Any, ClassVar, Literal

LABELS_ADDED: str
NODES_CREATED: str
NODES_DELETED: str
RELATIONSHIPS_DELETED: str
PROPERTIES_SET: str
RELATIONSHIPS_CREATED: str
INDICES_CREATED: str
INDICES_DELETED: str
CACHED_EXECUTION: str
INTERNAL_EXECUTION_TIME: str
STATS: Any

class ResultSetColumnTypes:
    COLUMN_UNKNOWN: ClassVar[Literal[0]]
    COLUMN_SCALAR: ClassVar[Literal[1]]
    COLUMN_NODE: ClassVar[Literal[2]]
    COLUMN_RELATION: ClassVar[Literal[3]]

class ResultSetScalarTypes:
    VALUE_UNKNOWN: ClassVar[Literal[0]]
    VALUE_NULL: ClassVar[Literal[1]]
    VALUE_STRING: ClassVar[Literal[2]]
    VALUE_INTEGER: ClassVar[Literal[3]]
    VALUE_BOOLEAN: ClassVar[Literal[4]]
    VALUE_DOUBLE: ClassVar[Literal[5]]
    VALUE_ARRAY: ClassVar[Literal[6]]
    VALUE_EDGE: ClassVar[Literal[7]]
    VALUE_NODE: ClassVar[Literal[8]]
    VALUE_PATH: ClassVar[Literal[9]]
    VALUE_MAP: ClassVar[Literal[10]]
    VALUE_POINT: ClassVar[Literal[11]]

class QueryResult:
    graph: Any
    header: Any
    result_set: Any
    def __init__(self, graph, response, profile: bool = False) -> None: ...
    def parse_results(self, raw_result_set) -> None: ...
    statistics: Any
    def parse_statistics(self, raw_statistics) -> None: ...
    def parse_header(self, raw_result_set): ...
    def parse_records(self, raw_result_set): ...
    def parse_entity_properties(self, props): ...
    def parse_string(self, cell): ...
    def parse_node(self, cell): ...
    def parse_edge(self, cell): ...
    def parse_path(self, cell): ...
    def parse_map(self, cell): ...
    def parse_point(self, cell): ...
    def parse_scalar(self, cell): ...
    def parse_profile(self, response) -> None: ...
    def is_empty(self): ...
    @property
    def labels_added(self): ...
    @property
    def nodes_created(self): ...
    @property
    def nodes_deleted(self): ...
    @property
    def properties_set(self): ...
    @property
    def relationships_created(self): ...
    @property
    def relationships_deleted(self): ...
    @property
    def indices_created(self): ...
    @property
    def indices_deleted(self): ...
    @property
    def cached_execution(self): ...
    @property
    def run_time_ms(self): ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.utils.backends import _dispatch

@_dispatch
def k_clique_communities(G, k, cliques: Incomplete | None = None) -> Generator[Incomplete, None, None]: ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import ChineseNewYearCalendar

class Taiwan(ChineseNewYearCalendar):
    FIXED_HOLIDAYS: Incomplete
    include_chinese_new_year_eve: ClassVar[bool]
    include_chinese_second_day: ClassVar[bool]
    def is_working_day(self, day, *args, **kwargs): ...
    def get_variable_days(self, year): ...



from _typeshed import Incomplete
from collections.abc import Container, Iterable, Iterator, Sequence
from re import Pattern
from typing import Any, Final
from typing_extensions import TypeAlias

from html5lib.filters.base import Filter
from html5lib.treewalkers.base import TreeWalker

from .callbacks import _Callback, _HTMLAttrs

DEFAULT_CALLBACKS: Final[list[_Callback]]
TLDS: Final[list[str]]

def build_url_re(tlds: Iterable[str] = ..., protocols: Iterable[str] = ...) -> Pattern[str]: ...

URL_RE: Final[Pattern[str]]
PROTO_RE: Final[Pattern[str]]

def build_email_re(tlds: Iterable[str] = ...) -> Pattern[str]: ...

EMAIL_RE: Final[Pattern[str]]

class Linker:
    def __init__(
        self,
        callbacks: Iterable[_Callback] = ...,
        skip_tags: Container[str] | None = None,
        parse_email: bool = False,
        url_re: Pattern[str] = ...,
        email_re: Pattern[str] = ...,
        recognized_tags: Container[str] | None = ...,
    ) -> None: ...
    def linkify(self, text: str) -> str: ...

# TODO: `_Token` might be converted into `TypedDict`
# or `html5lib` token might be reused
_Token: TypeAlias = dict[str, Any]

class LinkifyFilter(Filter):
    callbacks: Iterable[_Callback]
    skip_tags: Container[str]
    parse_email: bool
    url_re: Pattern[str]
    email_re: Pattern[str]
    def __init__(
        self,
        source: TreeWalker,
        callbacks: Iterable[_Callback] | None = ...,
        skip_tags: Container[str] | None = None,
        parse_email: bool = False,
        url_re: Pattern[str] = ...,
        email_re: Pattern[str] = ...,
    ) -> None: ...
    def apply_callbacks(self, attrs: _HTMLAttrs, is_new: bool) -> _HTMLAttrs | None: ...
    def extract_character_data(self, token_list: Iterable[_Token]) -> str: ...
    def handle_email_addresses(self, src_iter: Iterable[_Token]) -> Iterator[_Token]: ...
    def strip_non_url_bits(self, fragment: str) -> tuple[str, str, str]: ...
    def handle_links(self, src_iter: Iterable[_Token]) -> Iterator[_Token]: ...
    def handle_a_tag(self, token_buffer: Sequence[_Token]) -> Iterator[_Token]: ...
    def extract_entities(self, token: _Token) -> Iterator[_Token]: ...
    def __iter__(self) -> Iterator[Incomplete]: ...



from _typeshed import Incomplete
from typing import ClassVar

from ..core import ChineseNewYearCalendar, IslamicMixin

class Malaysia(IslamicMixin, ChineseNewYearCalendar):
    include_labour_day: ClassVar[bool]
    labour_day_label: ClassVar[str]
    include_nuzul_al_quran: ClassVar[bool]
    include_eid_al_fitr: ClassVar[bool]
    length_eid_al_fitr: int
    eid_al_fitr_label: ClassVar[str]
    include_day_of_sacrifice: ClassVar[bool]
    day_of_sacrifice_label: ClassVar[str]
    include_islamic_new_year: ClassVar[bool]
    include_prophet_birthday: ClassVar[bool]
    WEEKEND_DAYS: Incomplete
    FIXED_HOLIDAYS: Incomplete
    MSIA_DEEPAVALI: Incomplete
    MSIA_THAIPUSAM: Incomplete
    chinese_new_year_label: ClassVar[str]
    include_chinese_second_day: ClassVar[bool]
    chinese_second_day_label: ClassVar[str]
    shift_sunday_holidays: ClassVar[bool]
    def get_variable_days(self, year): ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def directed_edge_swap(G, *, nswap: int = 1, max_tries: int = 100, seed: Incomplete | None = None): ...
@_dispatch
def double_edge_swap(G, nswap: int = 1, max_tries: int = 100, seed: Incomplete | None = None): ...
@_dispatch
def connected_double_edge_swap(G, nswap: int = 1, _window_threshold: int = 3, seed: Incomplete | None = None): ...



from collections.abc import Callable, Container, Iterable, Mapping, Sequence
from typing import Any, Final, NamedTuple
from typing_extensions import Self, TypeAlias

from .version import __version__ as __version__

__all__ = ["tabulate", "tabulate_formats", "simple_separated_format"]

# These constants are meant to be configurable
# https://github.com/astanin/python-tabulate#text-formatting
PRESERVE_WHITESPACE: bool
MIN_PADDING: int
# https://github.com/astanin/python-tabulate#wide-fullwidth-cjk-symbols
WIDE_CHARS_MODE: bool
SEPARATING_LINE: str

class Line(NamedTuple):
    begin: str
    hline: str
    sep: str
    end: str

class DataRow(NamedTuple):
    begin: str
    sep: str
    end: str

_TableFormatLine: TypeAlias = None | Line | Callable[[list[int], list[str]], str]
_TableFormatRow: TypeAlias = None | DataRow | Callable[[list[Any], list[int], list[str]], str]

class TableFormat(NamedTuple):
    lineabove: _TableFormatLine
    linebelowheader: _TableFormatLine
    linebetweenrows: _TableFormatLine
    linebelow: _TableFormatLine
    headerrow: _TableFormatRow
    datarow: _TableFormatRow
    padding: int
    with_header_hide: Container[str] | None

LATEX_ESCAPE_RULES: Final[dict[str, str]]
tabulate_formats: list[str]
multiline_formats: dict[str, str]

def simple_separated_format(separator: str) -> TableFormat: ...
def tabulate(
    tabular_data: Mapping[str, Iterable[Any]] | Iterable[Iterable[Any]],
    headers: str | dict[str, str] | Sequence[str] = (),
    tablefmt: str | TableFormat = "simple",
    floatfmt: str | Iterable[str] = "g",
    intfmt: str | Iterable[str] = "",
    numalign: str | None = "default",
    stralign: str | None = "default",
    missingval: str | Iterable[str] = "",
    showindex: str | bool | Iterable[Any] = "default",
    disable_numparse: bool | Iterable[int] = False,
    colalign: Iterable[str | None] | None = None,
    maxcolwidths: int | Iterable[int | None] | None = None,
    rowalign: str | Iterable[str] | None = None,
    maxheadercolwidths: int | Iterable[int] | None = None,
) -> str: ...

class JupyterHTMLStr(str):
    @property
    def str(self) -> Self: ...



from _typeshed import Incomplete
from collections.abc import Generator
from typing import overload

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatch

@_dispatch
def has_path(G, source, target): ...
@overload
def shortest_path(
    G: Graph[_Node], source: _Node, target: _Node, weight: Incomplete | None = None, method: str = "dijkstra"
) -> list[_Node]: ...
@overload
def shortest_path(G: Graph[_Node], target: _Node, method: str = "dijkstra") -> dict[_Node, list[_Node]]: ...
@overload
def shortest_path(G: Graph[_Node], source: _Node, method: str = "dijkstra") -> dict[_Node, list[_Node]]: ...
@_dispatch
def shortest_path_length(
    G,
    source: Incomplete | None = None,
    target: Incomplete | None = None,
    weight: Incomplete | None = None,
    method: str = "dijkstra",
): ...
@_dispatch
def average_shortest_path_length(G, weight: Incomplete | None = None, method: str | None = None): ...
@_dispatch
def all_shortest_paths(
    G: Graph[_Node], source: _Node, target: _Node, weight: Incomplete | None = None, method: str = "dijkstra"
) -> Generator[list[_Node], None, None]: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def partial_duplication_graph(N, n, p, q, seed: Incomplete | None = None): ...
@_dispatch
def duplication_divergence_graph(n, p, seed: Incomplete | None = None): ...



from collections.abc import Iterable
from logging import Logger
from typing import Any

log: Logger
SUPPORTED_MODULES: Any
NO_DOUBLE_PATCH: Any

def patch_all(double_patch: bool = False) -> None: ...
def patch(modules_to_patch: Iterable[str], raise_errors: bool = True, ignore_module_patterns: str | None = None) -> None: ...



from _typeshed import Incomplete

from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Key(SystemBackendMixin):
    def read_root_generation_progress(self): ...
    def start_root_token_generation(self, otp: Incomplete | None = None, pgp_key: Incomplete | None = None): ...
    def generate_root(self, key, nonce): ...
    def cancel_root_generation(self): ...
    def get_encryption_key_status(self): ...
    def rotate_encryption_key(self): ...
    def read_rekey_progress(self, recovery_key: bool = False): ...
    def start_rekey(
        self,
        secret_shares: int = 5,
        secret_threshold: int = 3,
        pgp_keys: Incomplete | None = None,
        backup: bool = False,
        require_verification: bool = False,
        recovery_key: bool = False,
    ): ...
    def cancel_rekey(self, recovery_key: bool = False): ...
    def rekey(self, key, nonce: Incomplete | None = None, recovery_key: bool = False): ...
    def rekey_multi(self, keys, nonce: Incomplete | None = None, recovery_key: bool = False): ...
    def read_backup_keys(self, recovery_key: bool = False): ...
    def cancel_rekey_verify(self): ...
    def rekey_verify(self, key, nonce): ...
    def rekey_verify_multi(self, keys, nonce): ...
    def read_rekey_verify_progress(self): ...



from _typeshed import Incomplete

from hvac.api.system_backend.system_backend_mixin import SystemBackendMixin

class Quota(SystemBackendMixin):
    def read_quota(self, name): ...
    def list_quotas(self): ...
    def create_or_update_quota(
        self,
        name,
        rate,
        path: Incomplete | None = None,
        interval: Incomplete | None = None,
        block_interval: Incomplete | None = None,
        role: Incomplete | None = None,
        rate_limit_type: Incomplete | None = None,
        inheritable: Incomplete | None = None,
    ): ...
    def delete_quota(self, name): ...



from _typeshed import Incomplete

from .core import Calendar

class IsoRegistry:
    STANDARD_MODULES: Incomplete
    region_registry: dict[str, Calendar]
    def __init__(self, load_standard_modules: bool = True) -> None: ...
    def register(self, iso_code, cls) -> None: ...
    def load_module_from_items(self, module_name, items) -> None: ...
    def get(self, iso_code): ...
    def get_subregions(self, iso_code): ...
    def get_calendars(self, region_codes: Incomplete | None = None, include_subregions: bool = False): ...

registry: IsoRegistry



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

def write_dot(G, path) -> None: ...
@_dispatch
def read_dot(path): ...
@_dispatch
def from_pydot(P): ...
def to_pydot(N): ...
def graphviz_layout(G, prog: str = "neato", root: Incomplete | None = None): ...
def pydot_layout(G, prog: str = "neato", root: Incomplete | None = None): ...



from _typeshed import Incomplete

class Script:
    openapi_types: Incomplete
    attribute_map: Incomplete
    discriminator: Incomplete
    def __init__(
        self,
        id: Incomplete | None = None,
        name: Incomplete | None = None,
        description: Incomplete | None = None,
        org_id: Incomplete | None = None,
        script: Incomplete | None = None,
        language: Incomplete | None = None,
        url: Incomplete | None = None,
        created_at: Incomplete | None = None,
        updated_at: Incomplete | None = None,
    ) -> None: ...
    @property
    def id(self): ...
    @id.setter
    def id(self, id) -> None: ...
    @property
    def name(self): ...
    @name.setter
    def name(self, name) -> None: ...
    @property
    def description(self): ...
    @description.setter
    def description(self, description) -> None: ...
    @property
    def org_id(self): ...
    @org_id.setter
    def org_id(self, org_id) -> None: ...
    @property
    def script(self): ...
    @script.setter
    def script(self, script) -> None: ...
    @property
    def language(self): ...
    @language.setter
    def language(self, language) -> None: ...
    @property
    def url(self): ...
    @url.setter
    def url(self, url) -> None: ...
    @property
    def created_at(self): ...
    @created_at.setter
    def created_at(self, created_at) -> None: ...
    @property
    def updated_at(self): ...
    @updated_at.setter
    def updated_at(self, updated_at) -> None: ...
    def to_dict(self): ...
    def to_str(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



from _typeshed import Incomplete
from typing import Any

from .base import Client as Client

class ServiceApplicationClient(Client):
    grant_type: str
    private_key: Any
    subject: Any
    issuer: Any
    audience: Any
    def __init__(
        self,
        client_id,
        private_key: Incomplete | None = None,
        subject: Incomplete | None = None,
        issuer: Incomplete | None = None,
        audience: Incomplete | None = None,
        **kwargs,
    ) -> None: ...
    def prepare_request_body(
        self,
        private_key: Incomplete | None = None,
        subject: Incomplete | None = None,
        issuer: Incomplete | None = None,
        audience: Incomplete | None = None,
        expires_at: Incomplete | None = None,
        issued_at: Incomplete | None = None,
        extra_claims: Incomplete | None = None,
        body: str = "",
        scope: Incomplete | None = None,
        include_client_id: bool = False,
        **kwargs,
    ): ...



from _typeshed import Incomplete

ADS_ATTR_CLEAR: int
ADS_ATTR_UPDATE: int
ADS_ATTR_APPEND: int
ADS_ATTR_DELETE: int
ADS_EXT_MINEXTDISPID: int
ADS_EXT_MAXEXTDISPID: int
ADS_EXT_INITCREDENTIALS: int
ADS_EXT_INITIALIZE_COMPLETE: int
ADS_SEARCHPREF_ASYNCHRONOUS: int
ADS_SEARCHPREF_DEREF_ALIASES: int
ADS_SEARCHPREF_SIZE_LIMIT: int
ADS_SEARCHPREF_TIME_LIMIT: int
ADS_SEARCHPREF_ATTRIBTYPES_ONLY: int
ADS_SEARCHPREF_SEARCH_SCOPE: int
ADS_SEARCHPREF_TIMEOUT: int
ADS_SEARCHPREF_PAGESIZE: int
ADS_SEARCHPREF_PAGED_TIME_LIMIT: int
ADS_SEARCHPREF_CHASE_REFERRALS: int
ADS_SEARCHPREF_SORT_ON: int
ADS_SEARCHPREF_CACHE_RESULTS: int
ADS_SEARCHPREF_DIRSYNC: int
ADS_SEARCHPREF_TOMBSTONE: int
ADS_SCOPE_BASE: int
ADS_SCOPE_ONELEVEL: int
ADS_SCOPE_SUBTREE: int
ADS_SECURE_AUTHENTICATION: int
ADS_USE_ENCRYPTION: int
ADS_USE_SSL: int
ADS_READONLY_SERVER: int
ADS_PROMPT_CREDENTIALS: int
ADS_NO_AUTHENTICATION: int
ADS_FAST_BIND: int
ADS_USE_SIGNING: int
ADS_USE_SEALING: int
ADS_USE_DELEGATION: int
ADS_SERVER_BIND: int
ADSTYPE_INVALID: int
ADSTYPE_DN_STRING: Incomplete
ADSTYPE_CASE_EXACT_STRING: Incomplete
ADSTYPE_CASE_IGNORE_STRING: Incomplete
ADSTYPE_PRINTABLE_STRING: Incomplete
ADSTYPE_NUMERIC_STRING: Incomplete
ADSTYPE_BOOLEAN: Incomplete
ADSTYPE_INTEGER: Incomplete
ADSTYPE_OCTET_STRING: Incomplete
ADSTYPE_UTC_TIME: Incomplete
ADSTYPE_LARGE_INTEGER: Incomplete
ADSTYPE_PROV_SPECIFIC: Incomplete
ADSTYPE_OBJECT_CLASS: Incomplete
ADSTYPE_CASEIGNORE_LIST: Incomplete
ADSTYPE_OCTET_LIST: Incomplete
ADSTYPE_PATH: Incomplete
ADSTYPE_POSTALADDRESS: Incomplete
ADSTYPE_TIMESTAMP: Incomplete
ADSTYPE_BACKLINK: Incomplete
ADSTYPE_TYPEDNAME: Incomplete
ADSTYPE_HOLD: Incomplete
ADSTYPE_NETADDRESS: Incomplete
ADSTYPE_REPLICAPOINTER: Incomplete
ADSTYPE_FAXNUMBER: Incomplete
ADSTYPE_EMAIL: Incomplete
ADSTYPE_NT_SECURITY_DESCRIPTOR: Incomplete
ADSTYPE_UNKNOWN: Incomplete
ADSTYPE_DN_WITH_BINARY: Incomplete
ADSTYPE_DN_WITH_STRING: Incomplete
ADS_PROPERTY_CLEAR: int
ADS_PROPERTY_UPDATE: int
ADS_PROPERTY_APPEND: int
ADS_PROPERTY_DELETE: int
ADS_SYSTEMFLAG_DISALLOW_DELETE: int
ADS_SYSTEMFLAG_CONFIG_ALLOW_RENAME: int
ADS_SYSTEMFLAG_CONFIG_ALLOW_MOVE: int
ADS_SYSTEMFLAG_CONFIG_ALLOW_LIMITED_MOVE: int
ADS_SYSTEMFLAG_DOMAIN_DISALLOW_RENAME: int
ADS_SYSTEMFLAG_DOMAIN_DISALLOW_MOVE: int
ADS_SYSTEMFLAG_CR_NTDS_NC: int
ADS_SYSTEMFLAG_CR_NTDS_DOMAIN: int
ADS_SYSTEMFLAG_ATTR_NOT_REPLICATED: int
ADS_SYSTEMFLAG_ATTR_IS_CONSTRUCTED: int
ADS_GROUP_TYPE_GLOBAL_GROUP: int
ADS_GROUP_TYPE_DOMAIN_LOCAL_GROUP: int
ADS_GROUP_TYPE_LOCAL_GROUP: int
ADS_GROUP_TYPE_UNIVERSAL_GROUP: int
ADS_GROUP_TYPE_SECURITY_ENABLED: int
ADS_UF_SCRIPT: int
ADS_UF_ACCOUNTDISABLE: int
ADS_UF_HOMEDIR_REQUIRED: int
ADS_UF_LOCKOUT: int
ADS_UF_PASSWD_NOTREQD: int
ADS_UF_PASSWD_CANT_CHANGE: int
ADS_UF_ENCRYPTED_TEXT_PASSWORD_ALLOWED: int
ADS_UF_TEMP_DUPLICATE_ACCOUNT: int
ADS_UF_NORMAL_ACCOUNT: int
ADS_UF_INTERDOMAIN_TRUST_ACCOUNT: int
ADS_UF_WORKSTATION_TRUST_ACCOUNT: int
ADS_UF_SERVER_TRUST_ACCOUNT: int
ADS_UF_DONT_EXPIRE_PASSWD: int
ADS_UF_MNS_LOGON_ACCOUNT: int
ADS_UF_SMARTCARD_REQUIRED: int
ADS_UF_TRUSTED_FOR_DELEGATION: int
ADS_UF_NOT_DELEGATED: int
ADS_UF_USE_DES_KEY_ONLY: int
ADS_UF_DONT_REQUIRE_PREAUTH: int
ADS_UF_PASSWORD_EXPIRED: int
ADS_UF_TRUSTED_TO_AUTHENTICATE_FOR_DELEGATION: int
ADS_RIGHT_DELETE: int
ADS_RIGHT_READ_CONTROL: int
ADS_RIGHT_WRITE_DAC: int
ADS_RIGHT_WRITE_OWNER: int
ADS_RIGHT_SYNCHRONIZE: int
ADS_RIGHT_ACCESS_SYSTEM_SECURITY: int
ADS_RIGHT_GENERIC_READ: int
ADS_RIGHT_GENERIC_WRITE: int
ADS_RIGHT_GENERIC_EXECUTE: int
ADS_RIGHT_GENERIC_ALL: int
ADS_RIGHT_DS_CREATE_CHILD: int
ADS_RIGHT_DS_DELETE_CHILD: int
ADS_RIGHT_ACTRL_DS_LIST: int
ADS_RIGHT_DS_SELF: int
ADS_RIGHT_DS_READ_PROP: int
ADS_RIGHT_DS_WRITE_PROP: int
ADS_RIGHT_DS_DELETE_TREE: int
ADS_RIGHT_DS_LIST_OBJECT: int
ADS_RIGHT_DS_CONTROL_ACCESS: int
ADS_ACETYPE_ACCESS_ALLOWED: int
ADS_ACETYPE_ACCESS_DENIED: int
ADS_ACETYPE_SYSTEM_AUDIT: int
ADS_ACETYPE_ACCESS_ALLOWED_OBJECT: int
ADS_ACETYPE_ACCESS_DENIED_OBJECT: int
ADS_ACETYPE_SYSTEM_AUDIT_OBJECT: int
ADS_ACETYPE_SYSTEM_ALARM_OBJECT: int
ADS_ACETYPE_ACCESS_ALLOWED_CALLBACK: int
ADS_ACETYPE_ACCESS_DENIED_CALLBACK: int
ADS_ACETYPE_ACCESS_ALLOWED_CALLBACK_OBJECT: int
ADS_ACETYPE_ACCESS_DENIED_CALLBACK_OBJECT: int
ADS_ACETYPE_SYSTEM_AUDIT_CALLBACK: int
ADS_ACETYPE_SYSTEM_ALARM_CALLBACK: int
ADS_ACETYPE_SYSTEM_AUDIT_CALLBACK_OBJECT: int
ADS_ACETYPE_SYSTEM_ALARM_CALLBACK_OBJECT: int
ADS_ACEFLAG_INHERIT_ACE: int
ADS_ACEFLAG_NO_PROPAGATE_INHERIT_ACE: int
ADS_ACEFLAG_INHERIT_ONLY_ACE: int
ADS_ACEFLAG_INHERITED_ACE: int
ADS_ACEFLAG_VALID_INHERIT_FLAGS: int
ADS_ACEFLAG_SUCCESSFUL_ACCESS: int
ADS_ACEFLAG_FAILED_ACCESS: int
ADS_FLAG_OBJECT_TYPE_PRESENT: int
ADS_FLAG_INHERITED_OBJECT_TYPE_PRESENT: int
ADS_SD_CONTROL_SE_OWNER_DEFAULTED: int
ADS_SD_CONTROL_SE_GROUP_DEFAULTED: int
ADS_SD_CONTROL_SE_DACL_PRESENT: int
ADS_SD_CONTROL_SE_DACL_DEFAULTED: int
ADS_SD_CONTROL_SE_SACL_PRESENT: int
ADS_SD_CONTROL_SE_SACL_DEFAULTED: int
ADS_SD_CONTROL_SE_DACL_AUTO_INHERIT_REQ: int
ADS_SD_CONTROL_SE_SACL_AUTO_INHERIT_REQ: int
ADS_SD_CONTROL_SE_DACL_AUTO_INHERITED: int
ADS_SD_CONTROL_SE_SACL_AUTO_INHERITED: int
ADS_SD_CONTROL_SE_DACL_PROTECTED: int
ADS_SD_CONTROL_SE_SACL_PROTECTED: int
ADS_SD_CONTROL_SE_SELF_RELATIVE: int
ADS_SD_REVISION_DS: int
ADS_NAME_TYPE_1779: int
ADS_NAME_TYPE_CANONICAL: int
ADS_NAME_TYPE_NT4: int
ADS_NAME_TYPE_DISPLAY: int
ADS_NAME_TYPE_DOMAIN_SIMPLE: int
ADS_NAME_TYPE_ENTERPRISE_SIMPLE: int
ADS_NAME_TYPE_GUID: int
ADS_NAME_TYPE_UNKNOWN: int
ADS_NAME_TYPE_USER_PRINCIPAL_NAME: int
ADS_NAME_TYPE_CANONICAL_EX: int
ADS_NAME_TYPE_SERVICE_PRINCIPAL_NAME: int
ADS_NAME_TYPE_SID_OR_SID_HISTORY_NAME: int
ADS_NAME_INITTYPE_DOMAIN: int
ADS_NAME_INITTYPE_SERVER: int
ADS_NAME_INITTYPE_GC: int
ADS_OPTION_SERVERNAME: int
ADS_OPTION_REFERRALS: Incomplete
ADS_OPTION_PAGE_SIZE: Incomplete
ADS_OPTION_SECURITY_MASK: Incomplete
ADS_OPTION_MUTUAL_AUTH_STATUS: Incomplete
ADS_OPTION_QUOTA: Incomplete
ADS_OPTION_PASSWORD_PORTNUMBER: Incomplete
ADS_OPTION_PASSWORD_METHOD: Incomplete
ADS_SECURITY_INFO_OWNER: int
ADS_SECURITY_INFO_GROUP: int
ADS_SECURITY_INFO_DACL: int
ADS_SECURITY_INFO_SACL: int
ADS_SETTYPE_FULL: int
ADS_SETTYPE_PROVIDER: int
ADS_SETTYPE_SERVER: int
ADS_SETTYPE_DN: int
ADS_FORMAT_WINDOWS: int
ADS_FORMAT_WINDOWS_NO_SERVER: int
ADS_FORMAT_WINDOWS_DN: int
ADS_FORMAT_WINDOWS_PARENT: int
ADS_FORMAT_X500: int
ADS_FORMAT_X500_NO_SERVER: int
ADS_FORMAT_X500_DN: int
ADS_FORMAT_X500_PARENT: int
ADS_FORMAT_SERVER: int
ADS_FORMAT_PROVIDER: int
ADS_FORMAT_LEAF: int
ADS_DISPLAY_FULL: int
ADS_DISPLAY_VALUE_ONLY: int
ADS_ESCAPEDMODE_DEFAULT: int
ADS_ESCAPEDMODE_ON: int
ADS_ESCAPEDMODE_OFF: int
ADS_ESCAPEDMODE_OFF_EX: int
ADS_PATH_FILE: int
ADS_PATH_FILESHARE: int
ADS_PATH_REGISTRY: int
ADS_SD_FORMAT_IID: int
ADS_SD_FORMAT_RAW: int
ADS_SD_FORMAT_HEXSTRING: int
E_ADS_BAD_PATHNAME: Incomplete
E_ADS_INVALID_DOMAIN_OBJECT: Incomplete
E_ADS_INVALID_USER_OBJECT: Incomplete
E_ADS_INVALID_COMPUTER_OBJECT: Incomplete
E_ADS_UNKNOWN_OBJECT: Incomplete
E_ADS_PROPERTY_NOT_SET: Incomplete
E_ADS_PROPERTY_NOT_SUPPORTED: Incomplete
E_ADS_PROPERTY_INVALID: Incomplete
E_ADS_BAD_PARAMETER: Incomplete
E_ADS_OBJECT_UNBOUND: Incomplete
E_ADS_PROPERTY_NOT_MODIFIED: Incomplete
E_ADS_PROPERTY_MODIFIED: Incomplete
E_ADS_CANT_CONVERT_DATATYPE: Incomplete
E_ADS_PROPERTY_NOT_FOUND: Incomplete
E_ADS_OBJECT_EXISTS: Incomplete
E_ADS_SCHEMA_VIOLATION: Incomplete
E_ADS_COLUMN_NOT_SET: Incomplete
S_ADS_ERRORSOCCURRED: Incomplete
S_ADS_NOMORE_ROWS: Incomplete
S_ADS_NOMORE_COLUMNS: Incomplete
E_ADS_INVALID_FILTER: Incomplete
ADS_DEREF_NEVER: int
ADS_DEREF_SEARCHING: int
ADS_DEREF_FINDING: int
ADS_DEREF_ALWAYS: int
ADSIPROP_ASYNCHRONOUS: int
ADSIPROP_DEREF_ALIASES: int
ADSIPROP_SIZE_LIMIT: int
ADSIPROP_TIME_LIMIT: int
ADSIPROP_ATTRIBTYPES_ONLY: int
ADSIPROP_SEARCH_SCOPE: int
ADSIPROP_TIMEOUT: int
ADSIPROP_PAGESIZE: int
ADSIPROP_PAGED_TIME_LIMIT: int
ADSIPROP_CHASE_REFERRALS: int
ADSIPROP_SORT_ON: int
ADSIPROP_CACHE_RESULTS: int
ADSIPROP_ADSIFLAG: int
ADSI_DIALECT_LDAP: int
ADSI_DIALECT_SQL: int
ADS_CHASE_REFERRALS_NEVER: int
ADS_CHASE_REFERRALS_SUBORDINATE: int
ADS_CHASE_REFERRALS_EXTERNAL: int
ADS_CHASE_REFERRALS_ALWAYS: Incomplete
DSOP_SCOPE_TYPE_TARGET_COMPUTER: int
DSOP_SCOPE_TYPE_UPLEVEL_JOINED_DOMAIN: int
DSOP_SCOPE_TYPE_DOWNLEVEL_JOINED_DOMAIN: int
DSOP_SCOPE_TYPE_ENTERPRISE_DOMAIN: int
DSOP_SCOPE_TYPE_GLOBAL_CATALOG: int
DSOP_SCOPE_TYPE_EXTERNAL_UPLEVEL_DOMAIN: int
DSOP_SCOPE_TYPE_EXTERNAL_DOWNLEVEL_DOMAIN: int
DSOP_SCOPE_TYPE_WORKGROUP: int
DSOP_SCOPE_TYPE_USER_ENTERED_UPLEVEL_SCOPE: int
DSOP_SCOPE_TYPE_USER_ENTERED_DOWNLEVEL_SCOPE: int
DSOP_SCOPE_FLAG_STARTING_SCOPE: int
DSOP_SCOPE_FLAG_WANT_PROVIDER_WINNT: int
DSOP_SCOPE_FLAG_WANT_PROVIDER_LDAP: int
DSOP_SCOPE_FLAG_WANT_PROVIDER_GC: int
DSOP_SCOPE_FLAG_WANT_SID_PATH: int
DSOP_SCOPE_FLAG_WANT_DOWNLEVEL_BUILTIN_PATH: int
DSOP_SCOPE_FLAG_DEFAULT_FILTER_USERS: int
DSOP_SCOPE_FLAG_DEFAULT_FILTER_GROUPS: int
DSOP_SCOPE_FLAG_DEFAULT_FILTER_COMPUTERS: int
DSOP_SCOPE_FLAG_DEFAULT_FILTER_CONTACTS: int
DSOP_FILTER_INCLUDE_ADVANCED_VIEW: int
DSOP_FILTER_USERS: int
DSOP_FILTER_BUILTIN_GROUPS: int
DSOP_FILTER_WELL_KNOWN_PRINCIPALS: int
DSOP_FILTER_UNIVERSAL_GROUPS_DL: int
DSOP_FILTER_UNIVERSAL_GROUPS_SE: int
DSOP_FILTER_GLOBAL_GROUPS_DL: int
DSOP_FILTER_GLOBAL_GROUPS_SE: int
DSOP_FILTER_DOMAIN_LOCAL_GROUPS_DL: int
DSOP_FILTER_DOMAIN_LOCAL_GROUPS_SE: int
DSOP_FILTER_CONTACTS: int
DSOP_FILTER_COMPUTERS: int
DSOP_DOWNLEVEL_FILTER_USERS: int
DSOP_DOWNLEVEL_FILTER_LOCAL_GROUPS: int
DSOP_DOWNLEVEL_FILTER_GLOBAL_GROUPS: int
DSOP_DOWNLEVEL_FILTER_COMPUTERS: int
DSOP_DOWNLEVEL_FILTER_WORLD: int
DSOP_DOWNLEVEL_FILTER_AUTHENTICATED_USER: int
DSOP_DOWNLEVEL_FILTER_ANONYMOUS: int
DSOP_DOWNLEVEL_FILTER_BATCH: int
DSOP_DOWNLEVEL_FILTER_CREATOR_OWNER: int
DSOP_DOWNLEVEL_FILTER_CREATOR_GROUP: int
DSOP_DOWNLEVEL_FILTER_DIALUP: int
DSOP_DOWNLEVEL_FILTER_INTERACTIVE: int
DSOP_DOWNLEVEL_FILTER_NETWORK: int
DSOP_DOWNLEVEL_FILTER_SERVICE: int
DSOP_DOWNLEVEL_FILTER_SYSTEM: int
DSOP_DOWNLEVEL_FILTER_EXCLUDE_BUILTIN_GROUPS: int
DSOP_DOWNLEVEL_FILTER_TERMINAL_SERVER: int
DSOP_DOWNLEVEL_FILTER_ALL_WELLKNOWN_SIDS: int
DSOP_DOWNLEVEL_FILTER_LOCAL_SERVICE: int
DSOP_DOWNLEVEL_FILTER_NETWORK_SERVICE: int
DSOP_DOWNLEVEL_FILTER_REMOTE_LOGON: int
DSOP_FLAG_MULTISELECT: int
DSOP_FLAG_SKIP_TARGET_COMPUTER_DC_CHECK: int
CFSTR_DSOP_DS_SELECTION_LIST: str



from _typeshed import Incomplete

from networkx.utils.backends import _dispatch

@_dispatch
def average_neighbor_degree(
    G, source: str = "out", target: str = "out", nodes: Incomplete | None = None, weight: Incomplete | None = None
): ...



from _typeshed import StrOrBytesPath
from collections.abc import Callable
from contextlib import AbstractContextManager
from typing import Protocol
from typing_extensions import ParamSpec, Self, TypeAlias

from _cffi_backend import _CDataBase

from ._pygit2 import DiffFile, Oid
from .enums import CheckoutNotify, CheckoutStrategy, CredentialType, StashApplyProgress
from .remotes import TransferProgress
from .utils import _IntoStrArray

class Payload:
    def __init__(self, **kw: object) -> None: ...
    def check_error(self, error_code: int) -> None: ...

# Upstream is not yet defining a concrete type for certificates, and no usage example is
# available either.
_Certificate: TypeAlias = None

class _Credentials(Protocol):
    @property
    def credential_type(self) -> CredentialType: ...
    @property
    def credential_tuple(self) -> tuple[str, ...]: ...
    def __call__(self, _url: str, _username: str | None, _allowed: CredentialType) -> Self: ...

class RemoteCallbacks(Payload):
    # Upstream code is broken: the credentials() method is shadowed if the constructor
    # gets passed a non-None "credentials".
    # credentials: _Credentials | None
    certificate: _Certificate | None
    def __init__(self, credentials: _Credentials | None = None, certificate: _Certificate | None = None) -> None: ...
    def sideband_progress(self, string: str) -> None: ...
    def credentials(self, url: str, username_from_url: str | None, allowed_types: CredentialType) -> _Credentials: ...
    def certificate_check(self, certificate: _Certificate, valid: bool, host: str) -> bool: ...
    def transfer_progress(self, stats: TransferProgress) -> None: ...
    def update_tips(self, refname: str, old: Oid, new: Oid) -> None: ...
    def push_update_reference(self, refname: str, message: str) -> None: ...

class CheckoutCallbacks(Payload):
    def __init__(self) -> None: ...
    def checkout_notify_flags(self) -> CheckoutNotify: ...
    def checkout_notify(
        self, why: CheckoutNotify, path: str, baseline: DiffFile | None, target: DiffFile | None, workdir: DiffFile | None
    ) -> None: ...
    def checkout_progress(self, path: str, completed_steps: int, total_steps: int) -> None: ...

class StashApplyCallbacks(CheckoutCallbacks):
    def stash_apply_progress(self, progress: StashApplyProgress) -> None: ...

def git_clone_options(payload: Payload, opts: _CDataBase | None = None) -> AbstractContextManager[Payload]: ...
def git_fetch_options(payload: Payload, opts: _CDataBase | None = None) -> AbstractContextManager[Payload]: ...
def git_push_options(payload: Payload, opts: _CDataBase | None = None) -> AbstractContextManager[Payload]: ...
def git_remote_callbacks(payload: Payload) -> AbstractContextManager[Payload]: ...

_P = ParamSpec("_P")

def libgit2_callback(f: Callable[_P, int]) -> Callable[_P, int]: ...
def libgit2_callback_void(f: Callable[_P, None]) -> Callable[_P, None]: ...

_CredentialsFn: TypeAlias = Callable[[str | None, str | None, CredentialType], _Credentials]

def get_credentials(fn: _CredentialsFn, url: _CDataBase, username: _CDataBase, allowed: CredentialType) -> _CDataBase: ...
def git_checkout_options(
    callbacks: CheckoutCallbacks | None = None,
    strategy: CheckoutStrategy | None = None,
    directory: StrOrBytesPath | None = None,
    paths: _IntoStrArray = None,
) -> AbstractContextManager[Payload]: ...
def git_stash_apply_options(
    callbacks: StashApplyCallbacks | None = None,
    reinstate_index: bool = False,
    strategy: CheckoutStrategy | None = None,
    directory: StrOrBytesPath | None = None,
    paths: _IntoStrArray = None,
) -> AbstractContextManager[Payload]: ...



from _typeshed import StrOrBytesPath, SupportsAllComparisons
from collections.abc import Callable

# This is intentionally duplicated and placed above the _pygit2
# star-import to workaround mypy issue #16972, so that consumers
# of the stubs see the correct, wrapped type for the name
# "Repository".
from .repository import Repository as Repository  # isort: skip

from . import enums
from ._build import __version__ as __version__
from ._pygit2 import *  # type: ignore[assignment]
from .blame import Blame as Blame, BlameHunk as BlameHunk
from .blob import BlobIO as BlobIO
from .callbacks import (
    CheckoutCallbacks as CheckoutCallbacks,
    Payload as Payload,
    RemoteCallbacks,
    StashApplyCallbacks as StashApplyCallbacks,
    get_credentials as get_credentials,
)
from .config import Config as Config
from .credentials import *
from .errors import Passthrough as Passthrough
from .filter import Filter as Filter
from .index import Index as Index, IndexEntry as IndexEntry
from .legacyenums import *
from .packbuilder import PackBuilder as PackBuilder
from .remotes import Remote as Remote
from .repository import Repository as Repository  # noqa: F811 # intentional workaround
from .settings import Settings
from .submodules import Submodule as Submodule

features: enums.Feature
LIBGIT2_VER: tuple[int, int, int]

def init_repository(
    path: StrOrBytesPath | None,
    bare: bool = False,
    flags: enums.RepositoryInitFlag = ...,
    mode: int | enums.RepositoryInitMode = ...,
    workdir_path: str | None = None,
    description: str | None = None,
    template_path: str | None = None,
    initial_head: str | None = None,
    origin_url: str | None = None,
) -> Repository: ...
def clone_repository(
    url: str,
    path: str,
    bare: bool = False,
    repository: Callable[[str, bool], Repository] | None = None,
    remote: Callable[[Repository, str, str], Remote] | None = None,
    checkout_branch: str | None = None,
    callbacks: RemoteCallbacks | None = None,
    depth: int = 0,
) -> Repository: ...

tree_entry_key: Callable[[Object], SupportsAllComparisons]  # functools.cmp_to_key(tree_entry_cmp)
settings: Settings



import typing as t
from _typeshed import Incomplete

import click

__version__: str

class DefaultGroup(click.Group):
    ignore_unknown_options: bool
    default_cmd_name: str | None
    default_if_no_args: bool
    def __init__(self, *args, **kwargs) -> None: ...
    def set_default_command(self, command: click.Command) -> None: ...
    def parse_args(self, ctx: click.Context, args: list[str]) -> list[str]: ...
    def get_command(self, ctx: click.Context, cmd_name: str) -> click.Command | None: ...
    def resolve_command(self, ctx: click.Context, args: list[str]) -> tuple[str | None, click.Command | None, list[str]]: ...
    def format_commands(self, ctx: click.Context, formatter: click.HelpFormatter) -> None: ...
    def command(self, *args, **kwargs) -> click.Command: ...  # incomplete

class DefaultCommandFormatter:
    group: click.Group
    formatter: click.HelpFormatter
    mark: str
    def __init__(self, group: click.Group, formatter: click.HelpFormatter, mark: str = ...) -> None: ...
    def write_dl(self, rows: t.Sequence[tuple[str, str]], col_max: int = 30, col_spacing: int = -2) -> None: ...
    def __getattr__(self, attr: str) -> Incomplete: ...
    # __getattr__ used to ala-derive from click.HelpFormatter:
    # indent_increment: int
    # width: int | None
    # current_indent: int
    # buffer: t.List[str]
    # def write(self, string: str) -> None: ...
    # def indent(self) -> None: ...
    # def dedent(self) -> None: ...
    # def write_usage(self, prog: str, args: str = ..., prefix: str | None = ...) -> None: ...
    # def write_heading(self, heading: str) -> None: ...
    # def write_paragraph(self) -> None: ...
    # def write_text(self, text: str) -> None: ...
    # def section(self, name: str) -> t.Iterator[None]: ...
    # def indentation(self) -> t.Iterator[None]: ...
    # def getvalue(self) -> str: ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.utils.backends import _dispatch

@_dispatch
def nonisomorphic_trees(order, create: str = "graph") -> Generator[Incomplete, None, None]: ...
@_dispatch
def number_of_nonisomorphic_trees(order): ...



from collections.abc import Iterable

import _win32typing
from win32.lib.pywintypes import error as error

def CancelWaitableTimer() -> None: ...
def CreateEvent(
    EventAttributes: _win32typing.PySECURITY_ATTRIBUTES | None,
    bManualReset: int | bool,
    bInitialState: int | bool,
    Name: str | None,
    /,
) -> int: ...
def CreateMutex(MutexAttributes: _win32typing.PySECURITY_ATTRIBUTES, InitialOwner, Name: str, /) -> int: ...
def CreateSemaphore(
    SemaphoreAttributes: _win32typing.PySECURITY_ATTRIBUTES, InitialCount, MaximumCount, SemaphoreName, /
) -> int: ...
def CreateWaitableTimer(TimerAttributes: _win32typing.PySECURITY_ATTRIBUTES, ManualReset, TimerName, /) -> int: ...
def CreateWaitableTimerEx(
    lpTimerAttributes: _win32typing.PySECURITY_ATTRIBUTES | None, lpTimerName: str | None, dwFlags: int, dwDesiredAccess: int, /
) -> _win32typing.PyHANDLE: ...
def MsgWaitForMultipleObjects(handlelist: Iterable[int], bWaitAll: int, milliseconds: int, wakeMask: int, /) -> int: ...
def MsgWaitForMultipleObjectsEx(handlelist: list[int], milliseconds, wakeMask, waitFlags, /): ...
def OpenEvent(desiredAccess, bInheritHandle, name: str, /) -> int: ...
def OpenMutex(desiredAccess, bInheritHandle, name: str, /) -> int: ...
def OpenSemaphore(desiredAccess, bInheritHandle, name: str, /) -> int: ...
def OpenWaitableTimer(desiredAccess, bInheritHandle, timerName, /) -> int: ...
def PulseEvent(hEvent: int, /) -> None: ...
def ReleaseMutex(hEvent: int, /) -> None: ...
def ReleaseSemaphore(hEvent: int, lReleaseCount, /): ...
def ResetEvent(hEvent: int, /) -> None: ...
def SetEvent(hEvent: int, /) -> None: ...
def SetWaitableTimer(handle: int, dueTime, period, func, param, resume_state, /) -> None: ...
def WaitForMultipleObjects(handlelist: list[int], bWaitAll, milliseconds, /): ...
def WaitForMultipleObjectsEx(handlelist: list[int], bWaitAll, milliseconds, bAlertable, /): ...
def WaitForSingleObject(hHandle: int, milliseconds: int, /) -> int: ...
def WaitForSingleObjectEx(hHandle: int, milliseconds, bAlertable, /): ...
def WaitForInputIdle(hProcess: int, milliseconds, /): ...
def SignalObjectAndWait(*args): ...  # incomplete

CREATE_WAITABLE_TIMER_HIGH_RESOLUTION: int
CREATE_WAITABLE_TIMER_MANUAL_RESET: int
EVENT_ALL_ACCESS: int
EVENT_MODIFY_STATE: int
INFINITE: int
MAXIMUM_WAIT_OBJECTS: int
QS_ALLEVENTS: int
QS_ALLINPUT: int
QS_HOTKEY: int
QS_INPUT: int
QS_KEY: int
QS_MOUSE: int
QS_MOUSEBUTTON: int
QS_MOUSEMOVE: int
QS_PAINT: int
QS_POSTMESSAGE: int
QS_SENDMESSAGE: int
QS_TIMER: int
SYNCHRONIZE: int
TIMER_ALL_ACCESS: int
TIMER_MODIFY_STATE: int
TIMER_QUERY_STATE: int
WAIT_ABANDONED: int
WAIT_ABANDONED_0: int
WAIT_FAILED: int
WAIT_IO_COMPLETION: int
WAIT_OBJECT_0: int
WAIT_TIMEOUT: int
UNICODE: int



from collections.abc import Callable

from .client import Client, MQTTMessage, _UserData
from .publish import _TLS, _Auth, _Msg, _Proxy

def callback(
    callback: Callable[[Client, _UserData, MQTTMessage], None],
    topics: list[str],
    qos: int = 0,
    userdata: _UserData | None = None,
    hostname: str = "localhost",
    port: int = 1883,
    client_id: str = "",
    keepalive: int = 60,
    will: _Msg | None = None,
    auth: _Auth | None = None,
    tls: _TLS | None = None,
    protocol: int = 4,
    transport: str = "tcp",
    clean_session: bool = True,
    proxy_args: _Proxy | None = None,
) -> None: ...
def simple(
    topics: str | list[str],
    qos: int = 0,
    msg_count: int = 1,
    retained: bool = True,
    hostname: str = "localhost",
    port: int = 1883,
    client_id: str = "",
    keepalive: int = 60,
    will: _Msg | None = None,
    auth: _Auth | None = None,
    tls: _TLS | None = None,
    protocol: int = 4,
    transport: str = "tcp",
    clean_session: bool = True,
    proxy_args: _Proxy | None = None,
) -> list[MQTTMessage] | MQTTMessage: ...



from _typeshed import Incomplete
from typing import Any

class UnknownBackendError(ValueError):
    hasher: Any
    backend: Any
    def __init__(self, hasher, backend) -> None: ...

class MissingBackendError(RuntimeError): ...
class InternalBackendError(RuntimeError): ...
class PasswordValueError(ValueError): ...

class PasswordSizeError(PasswordValueError):
    max_size: Any
    def __init__(self, max_size, msg: Incomplete | None = None) -> None: ...

class PasswordTruncateError(PasswordSizeError):
    def __init__(self, cls, msg: Incomplete | None = None) -> None: ...

class PasslibSecurityError(RuntimeError): ...

class TokenError(ValueError):
    def __init__(self, msg: Incomplete | None = None, *args, **kwds) -> None: ...

class MalformedTokenError(TokenError): ...
class InvalidTokenError(TokenError): ...

class UsedTokenError(TokenError):
    expire_time: Any
    def __init__(self, *args, **kwds) -> None: ...

class UnknownHashError(ValueError):
    value: Any
    message: Any
    def __init__(self, message: Incomplete | None = None, value: Incomplete | None = None) -> None: ...

class PasslibWarning(UserWarning): ...
class PasslibConfigWarning(PasslibWarning): ...
class PasslibHashWarning(PasslibWarning): ...
class PasslibRuntimeWarning(PasslibWarning): ...
class PasslibSecurityWarning(PasslibWarning): ...

def type_name(value): ...
def ExpectedTypeError(value, expected, param): ...
def ExpectedStringError(value, param): ...
def MissingDigestError(handler: Incomplete | None = None): ...
def NullPasswordError(handler: Incomplete | None = None): ...
def InvalidHashError(handler: Incomplete | None = None): ...
def MalformedHashError(handler: Incomplete | None = None, reason: Incomplete | None = None): ...
def ZeroPaddedRoundsError(handler: Incomplete | None = None): ...
def ChecksumSizeError(handler, raw: bool = False): ...

ENABLE_DEBUG_ONLY_REPR: bool

def debug_only_repr(value, param: str = "hash"): ...
def CryptBackendError(handler, config, hash, source: str = "crypt.crypt()") -> None: ...



import time
from _typeshed import Incomplete
from types import TracebackType
from typing import Any

from ...core import AWSXRayRecorder
from .entity import Entity
from .segment import Segment

SUBSEGMENT_RECORDING_ATTRIBUTE: str

def set_as_recording(decorated_func, wrapped) -> None: ...
def is_already_recording(func): ...
def subsegment_decorator(wrapped, instance, args, kwargs): ...

class SubsegmentContextManager:
    name: str | None
    subsegment_kwargs: dict[str, Any] | None
    recorder: AWSXRayRecorder
    subsegment: Subsegment
    def __init__(self, recorder: AWSXRayRecorder, name: Incomplete | None = None, **subsegment_kwargs) -> None: ...
    def __call__(self, wrapped, instance, args: list[Any], kwargs: dict[str, Any]): ...
    def __enter__(self) -> Subsegment: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...

class Subsegment(Entity):
    parent_segment: Segment
    trace_id: str
    type: str
    namespace: str
    sql: dict[str, Any]
    def __init__(self, name: str, namespace: str, segment: Segment) -> None: ...
    def add_subsegment(self, subsegment: Subsegment) -> None: ...
    def remove_subsegment(self, subsegment: Subsegment) -> None: ...
    def close(self, end_time: time.struct_time | None = None) -> None: ...
    def set_sql(self, sql: dict[str, Any]) -> None: ...
    def to_dict(self) -> dict[str, Any]: ...
